import torch
from torch.utils.data import DataLoader, SequentialSampler
import transformers
from transformers import AutoTokenizer, squad_convert_examples_to_features
from transformers.data.processors.squad import SquadV1Processor
import os

def preprocess(json_file, batch_size, max_seq_length):
    ###preprocess data
    model_path = os.environ.get('MODEL_PATH')+ "/bert-squad-training"
    tokenizer = AutoTokenizer.from_pretrained(model_path, do_lower_case = False)
    squad_processor = SquadV1Processor()
    examples = squad_processor.get_dev_examples("", filename=json_file)
    features, dataset = squad_convert_examples_to_features(
        examples = examples,
        tokenizer = tokenizer,
        max_seq_length = max_seq_length,
        doc_stride = 128,
        max_query_length = 64,
        is_training = False,
        return_dataset = "pt",
        threads = 4)
    eval_sampler = SequentialSampler(dataset)
    eval_dataloader = DataLoader(dataset, sampler = eval_sampler, batch_size = batch_size, drop_last = False)
    print("Num examples = ", len(dataset))
    print("Batch size = ", batch_size)
    print("Iterations = ", len(eval_dataloader))
    return tokenizer, examples, features, eval_dataloader



import logging.handlers


class Logger(logging.Logger):
    def __init__(self, filename=None):
        super(Logger, self).__init__(self)
        cur_stream = logging.StreamHandler()
        cur_stream.setLevel(logging.DEBUG)
        formater = logging.Formatter(
            "%(asctime)s: %(levelname)s: %(filename)s:%(lineno)d] %(message)s"
        )
        cur_stream.setFormatter(formater)
        self.addHandler(cur_stream)


if __name__ == "__main__":
    pass



# Copyright (c) 2021 Mobvoi Inc. (authors: Binbin Zhang)
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import logging
import json
import random
import re
import tarfile
from subprocess import PIPE, Popen
from urllib.parse import urlparse

import torch
import torchaudio
import torchaudio.compliance.kaldi as kaldi
from torch.nn.utils.rnn import pad_sequence

AUDIO_FORMAT_SETS = set(['flac', 'mp3', 'm4a', 'ogg', 'opus', 'wav', 'wma'])


def url_opener(data):
    """ Give url or local file, return file descriptor
        Inplace operation.

        Args:
            data(Iterable[str]): url or local file list

        Returns:
            Iterable[{src, stream}]
    """
    for sample in data:
        assert 'src' in sample
        # TODO(Binbin Zhang): support HTTP
        url = sample['src']
        try:
            pr = urlparse(url)
            # local file
            if pr.scheme == '' or pr.scheme == 'file':
                stream = open(url, 'rb')
            # network file, such as HTTP(HDFS/OSS/S3)/HTTPS/SCP
            else:
                cmd = f'curl -s -L {url}'
                process = Popen(cmd, shell=True, stdout=PIPE)
                sample.update(process=process)
                stream = process.stdout
            sample.update(stream=stream)
            yield sample
        except Exception as ex:
            logging.warning('Failed to open {}'.format(url))


def tar_file_and_group(data):
    """ Expand a stream of open tar files into a stream of tar file contents.
        And groups the file with same prefix

        Args:
            data: Iterable[{src, stream}]

        Returns:
            Iterable[{key, wav, txt, sample_rate}]
    """
    for sample in data:
        assert 'stream' in sample
        stream = tarfile.open(fileobj=sample['stream'], mode="r|*")
        prev_prefix = None
        example = {}
        valid = True
        for tarinfo in stream:
            name = tarinfo.name
            pos = name.rfind('.')
            assert pos > 0
            prefix, postfix = name[:pos], name[pos + 1:]
            if prev_prefix is not None and prefix != prev_prefix:
                example['key'] = prev_prefix
                if valid:
                    yield example
                example = {}
                valid = True
            with stream.extractfile(tarinfo) as file_obj:
                try:
                    if postfix == 'txt':
                        example['txt'] = file_obj.read().decode('utf8').strip()
                    elif postfix in AUDIO_FORMAT_SETS:
                        waveform, sample_rate = torchaudio.load(file_obj)
                        example['wav'] = waveform
                        example['sample_rate'] = sample_rate
                    else:
                        example[postfix] = file_obj.read()
                except Exception as ex:
                    valid = False
                    logging.warning('error to parse {}'.format(name))
            prev_prefix = prefix
        if prev_prefix is not None:
            example['key'] = prev_prefix
            yield example
        stream.close()
        if 'process' in sample:
            sample['process'].communicate()
        sample['stream'].close()


def parse_raw(data):
    """ Parse key/wav/txt from json line

        Args:
            data: Iterable[str], str is a json line has key/wav/txt

        Returns:
            Iterable[{key, wav, txt, sample_rate}]
    """
    for sample in data:
        assert 'src' in sample
        json_line = sample['src']
        obj = json.loads(json_line)
        assert 'key' in obj
        assert 'wav' in obj
        assert 'txt' in obj
        key = obj['key']
        wav_file = obj['wav']
        txt = obj['txt']
        try:
            if 'start' in obj:
                assert 'end' in obj
                sample_rate = torchaudio.backend.sox_io_backend.info(
                    wav_file).sample_rate
                start_frame = int(obj['start'] * sample_rate)
                end_frame = int(obj['end'] * sample_rate)
                waveform, _ = torchaudio.backend.sox_io_backend.load(
                    filepath=wav_file,
                    num_frames=end_frame - start_frame,
                    frame_offset=start_frame)
            else:
                waveform, sample_rate = torchaudio.load(wav_file)
            example = dict(key=key,
                           txt=txt,
                           wav=waveform,
                           sample_rate=sample_rate)
            yield example
        except Exception as ex:
            logging.warning('Failed to read {}'.format(wav_file))


def filter(data,
           max_length=10240,
           min_length=10,
           token_max_length=200,
           token_min_length=1,
           min_output_input_ratio=0.0005,
           max_output_input_ratio=1):
    """ Filter sample according to feature and label length
        Inplace operation.

        Args::
            data: Iterable[{key, wav, label, sample_rate}]
            max_length: drop utterance which is greater than max_length(10ms)
            min_length: drop utterance which is less than min_length(10ms)
            token_max_length: drop utterance which is greater than
                token_max_length, especially when use char unit for
                english modeling
            token_min_length: drop utterance which is
                less than token_max_length
            min_output_input_ratio: minimal ration of
                token_length / feats_length(10ms)
            max_output_input_ratio: maximum ration of
                token_length / feats_length(10ms)

        Returns:
            Iterable[{key, wav, label, sample_rate}]
    """
    for sample in data:
        assert 'sample_rate' in sample
        assert 'wav' in sample
        assert 'label' in sample
        # sample['wav'] is torch.Tensor, we have 100 frames every second
        num_frames = sample['wav'].size(1) / sample['sample_rate'] * 100
        if num_frames < min_length:
            continue
        if num_frames > max_length:
            continue
        if len(sample['label']) < token_min_length:
            continue
        if len(sample['label']) > token_max_length:
            continue
        if num_frames != 0:
            if len(sample['label']) / num_frames < min_output_input_ratio:
                continue
            if len(sample['label']) / num_frames > max_output_input_ratio:
                continue
        yield sample


def resample(data, resample_rate=16000):
    """ Resample data.
        Inplace operation.

        Args:
            data: Iterable[{key, wav, label, sample_rate}]
            resample_rate: target resample rate

        Returns:
            Iterable[{key, wav, label, sample_rate}]
    """
    for sample in data:
        assert 'sample_rate' in sample
        assert 'wav' in sample
        sample_rate = sample['sample_rate']
        waveform = sample['wav']
        if sample_rate != resample_rate:
            sample['sample_rate'] = resample_rate
            sample['wav'] = torchaudio.transforms.Resample(
                orig_freq=sample_rate, new_freq=resample_rate)(waveform)
        yield sample


def speed_perturb(data, speeds=None):
    """ Apply speed perturb to the data.
        Inplace operation.

        Args:
            data: Iterable[{key, wav, label, sample_rate}]
            speeds(List[float]): optional speed

        Returns:
            Iterable[{key, wav, label, sample_rate}]
    """
    if speeds is None:
        speeds = [0.9, 1.0, 1.1]
    for sample in data:
        assert 'sample_rate' in sample
        assert 'wav' in sample
        sample_rate = sample['sample_rate']
        waveform = sample['wav']
        speed = random.choice(speeds)
        if speed != 1.0:
            wav, _ = torchaudio.sox_effects.apply_effects_tensor(
                waveform, sample_rate,
                [['speed', str(speed)], ['rate', str(sample_rate)]])
            sample['wav'] = wav

        yield sample


def compute_fbank(data,
                  num_mel_bins=23,
                  frame_length=25,
                  frame_shift=10,
                  dither=0.0):
    """ Extract fbank

        Args:
            data: Iterable[{key, wav, label, sample_rate}]

        Returns:
            Iterable[{key, feat, label}]
    """
    for sample in data:
        assert 'sample_rate' in sample
        assert 'wav' in sample
        assert 'key' in sample
        assert 'label' in sample
        sample_rate = sample['sample_rate']
        waveform = sample['wav']
        waveform = waveform * (1 << 15)
        # Only keep key, feat, label
        mat = kaldi.fbank(waveform,
                          num_mel_bins=num_mel_bins,
                          frame_length=frame_length,
                          frame_shift=frame_shift,
                          dither=dither,
                          energy_floor=0.0,
                          sample_frequency=sample_rate)
        yield dict(key=sample['key'], label=sample['label'], feat=mat)


def compute_mfcc(data,
                 num_mel_bins=23,
                 frame_length=25,
                 frame_shift=10,
                 dither=0.0,
                 num_ceps=40,
                 high_freq=0.0,
                 low_freq=20.0):
    """ Extract mfcc

        Args:
            data: Iterable[{key, wav, label, sample_rate}]

        Returns:
            Iterable[{key, feat, label}]
    """
    for sample in data:
        assert 'sample_rate' in sample
        assert 'wav' in sample
        assert 'key' in sample
        assert 'label' in sample
        sample_rate = sample['sample_rate']
        waveform = sample['wav']
        waveform = waveform * (1 << 15)
        # Only keep key, feat, label
        mat = kaldi.mfcc(waveform,
                         num_mel_bins=num_mel_bins,
                         frame_length=frame_length,
                         frame_shift=frame_shift,
                         dither=dither,
                         num_ceps=num_ceps,
                         high_freq=high_freq,
                         low_freq=low_freq,
                         sample_frequency=sample_rate)
        yield dict(key=sample['key'], label=sample['label'], feat=mat)


def __tokenize_by_bpe_model(sp, txt):
    tokens = []
    # CJK(China Japan Korea) unicode range is [U+4E00, U+9FFF], ref:
    # https://en.wikipedia.org/wiki/CJK_Unified_Ideographs_(Unicode_block)
    pattern = re.compile(r'([\u4e00-\u9fff])')
    # Example:
    #   txt   = "你好 ITS'S OKAY 的"
    #   chars = ["你", "好", " ITS'S OKAY ", "的"]
    chars = pattern.split(txt.upper())
    mix_chars = [w for w in chars if len(w.strip()) > 0]
    for ch_or_w in mix_chars:
        # ch_or_w is a single CJK charater(i.e., "你"), do nothing.
        if pattern.fullmatch(ch_or_w) is not None:
            tokens.append(ch_or_w)
        # ch_or_w contains non-CJK charaters(i.e., " IT'S OKAY "),
        # encode ch_or_w using bpe_model.
        else:
            for p in sp.encode_as_pieces(ch_or_w):
                tokens.append(p)

    return tokens


def tokenize(data,
             symbol_table,
             bpe_model=None,
             non_lang_syms=None,
             split_with_space=False):
    """ Decode text to chars or BPE
        Inplace operation

        Args:
            data: Iterable[{key, wav, txt, sample_rate}]

        Returns:
            Iterable[{key, wav, txt, tokens, label, sample_rate}]
    """
    if non_lang_syms is not None:
        non_lang_syms_pattern = re.compile(r"(\[[^\[\]]+\]|<[^<>]+>|{[^{}]+})")
    else:
        non_lang_syms = {}
        non_lang_syms_pattern = None

    if bpe_model is not None:
        import sentencepiece as spm
        sp = spm.SentencePieceProcessor()
        sp.load(bpe_model)
    else:
        sp = None

    for sample in data:
        assert 'txt' in sample
        txt = sample['txt'].strip()
        if non_lang_syms_pattern is not None:
            parts = non_lang_syms_pattern.split(txt.upper())
            parts = [w for w in parts if len(w.strip()) > 0]
        else:
            parts = [txt]

        label = []
        tokens = []
        for part in parts:
            if part in non_lang_syms:
                tokens.append(part)
            else:
                if bpe_model is not None:
                    tokens.extend(__tokenize_by_bpe_model(sp, part))
                else:
                    if split_with_space:
                        part = part.split(" ")
                    for ch in part:
                        if ch == ' ':
                            ch = "▁"
                        tokens.append(ch)

        for ch in tokens:
            if ch in symbol_table:
                label.append(symbol_table[ch])
            elif '<unk>' in symbol_table:
                label.append(symbol_table['<unk>'])

        sample['tokens'] = tokens
        sample['label'] = label
        yield sample


def spec_aug(data, num_t_mask=2, num_f_mask=2, max_t=50, max_f=10, max_w=80):
    """ Do spec augmentation
        Inplace operation

        Args:
            data: Iterable[{key, feat, label}]
            num_t_mask: number of time mask to apply
            num_f_mask: number of freq mask to apply
            max_t: max width of time mask
            max_f: max width of freq mask
            max_w: max width of time warp

        Returns
            Iterable[{key, feat, label}]
    """
    for sample in data:
        assert 'feat' in sample
        x = sample['feat']
        assert isinstance(x, torch.Tensor)
        y = x.clone().detach()
        max_frames = y.size(0)
        max_freq = y.size(1)
        # time mask
        for i in range(num_t_mask):
            start = random.randint(0, max_frames - 1)
            length = random.randint(1, max_t)
            end = min(max_frames, start + length)
            y[start:end, :] = 0
        # freq mask
        for i in range(num_f_mask):
            start = random.randint(0, max_freq - 1)
            length = random.randint(1, max_f)
            end = min(max_freq, start + length)
            y[:, start:end] = 0
        sample['feat'] = y
        yield sample


def spec_sub(data, max_t=20, num_t_sub=3):
    """ Do spec substitute
        Inplace operation

        Args:
            data: Iterable[{key, feat, label}]
            max_t: max width of time substitute
            num_t_sub: number of time substitute to apply

        Returns
            Iterable[{key, feat, label}]
    """
    for sample in data:
        assert 'feat' in sample
        x = sample['feat']
        assert isinstance(x, torch.Tensor)
        y = x.clone().detach()
        max_frames = y.size(0)
        for i in range(num_t_sub):
            start = random.randint(0, max_frames - 1)
            length = random.randint(1, max_t)
            end = min(max_frames, start + length)
            # only substitute the earlier time chosen randomly for current time
            pos = random.randint(0, start)
            y[start:end, :] = x[start - pos:end - pos, :]
        sample['feat'] = y
        yield sample


def shuffle(data, shuffle_size=10000):
    """ Local shuffle the data

        Args:
            data: Iterable[{key, feat, label}]
            shuffle_size: buffer size for shuffle

        Returns:
            Iterable[{key, feat, label}]
    """
    buf = []
    for sample in data:
        buf.append(sample)
        if len(buf) >= shuffle_size:
            random.shuffle(buf)
            for x in buf:
                yield x
            buf = []
    # The sample left over
    random.shuffle(buf)
    for x in buf:
        yield x


def sort(data, sort_size=500):
    """ Sort the data by feature length.
        Sort is used after shuffle and before batch, so we can group
        utts with similar lengths into a batch, and `sort_size` should
        be less than `shuffle_size`

        Args:
            data: Iterable[{key, feat, label}]
            sort_size: buffer size for sort

        Returns:
            Iterable[{key, feat, label}]
    """

    buf = []
    for sample in data:
        buf.append(sample)
        if len(buf) >= sort_size:
            buf.sort(key=lambda x: x['feat'].size(0))
            for x in buf:
                yield x
            buf = []
    # The sample left over
    buf.sort(key=lambda x: x['feat'].size(0))
    for x in buf:
        yield x


def static_batch(data, batch_size=16):
    """ Static batch the data by `batch_size`

        Args:
            data: Iterable[{key, feat, label}]
            batch_size: batch size

        Returns:
            Iterable[List[{key, feat, label}]]
    """
    buf = []
    for sample in data:
        buf.append(sample)
        if len(buf) >= batch_size:
            yield buf
            buf = []
    if len(buf) > 0:
        yield buf


def dynamic_batch(data, max_frames_in_batch=12000):
    """ Dynamic batch the data until the total frames in batch
        reach `max_frames_in_batch`

        Args:
            data: Iterable[{key, feat, label}]
            max_frames_in_batch: max_frames in one batch

        Returns:
            Iterable[List[{key, feat, label}]]
    """
    buf = []
    longest_frames = 0
    for sample in data:
        assert 'feat' in sample
        assert isinstance(sample['feat'], torch.Tensor)
        new_sample_frames = sample['feat'].size(0)
        longest_frames = max(longest_frames, new_sample_frames)
        frames_after_padding = longest_frames * (len(buf) + 1)
        if frames_after_padding > max_frames_in_batch:
            yield buf
            buf = [sample]
            longest_frames = new_sample_frames
        else:
            buf.append(sample)
    if len(buf) > 0:
        yield buf


def batch(data, batch_type='static', batch_size=16, max_frames_in_batch=12000):
    """ Wrapper for static/dynamic batch
    """
    if batch_type == 'static':
        return static_batch(data, batch_size)
    elif batch_type == 'dynamic':
        return dynamic_batch(data, max_frames_in_batch)
    else:
        logging.fatal('Unsupported batch type {}'.format(batch_type))


def padding(data):
    """ Padding the data into training data

        Args:
            data: Iterable[List[{key, feat, label}]]

        Returns:
            Iterable[Tuple(keys, feats, labels, feats lengths, label lengths)]
    """
    for sample in data:
        assert isinstance(sample, list)
        feats_length = torch.tensor([x['feat'].size(0) for x in sample],
                                    dtype=torch.int32)
        order = torch.argsort(feats_length, descending=True)
        feats_lengths = torch.tensor(
            [sample[i]['feat'].size(0) for i in order], dtype=torch.int32)
        sorted_feats = [sample[i]['feat'] for i in order]
        sorted_keys = [sample[i]['key'] for i in order]
        sorted_labels = [
            torch.tensor(sample[i]['label'], dtype=torch.int64) for i in order
        ]
        label_lengths = torch.tensor([x.size(0) for x in sorted_labels],
                                     dtype=torch.int32)

        padded_feats = pad_sequence(sorted_feats,
                                    batch_first=True,
                                    padding_value=0)
        padding_labels = pad_sequence(sorted_labels,
                                      batch_first=True,
                                      padding_value=-1)

        yield (sorted_keys, padded_feats, padding_labels, feats_lengths,
               label_lengths)



import magicmind.python.runtime as mm
from magicmind.python.runtime.parser import Parser
import numpy as np
import glob
import os 
import cv2

class CalibData(mm.CalibDataInterface):
    def __init__(self, shape: mm.Dims, index: int, max_samples: int, img_dir: str, pad_value):
        super().__init__()
        assert os.path.isdir(img_dir)
        self.data_paths_ = glob.glob(img_dir + '/*.jpg')
        print("calibrate samples : ", len(self.data_paths_))
        self.shape_ = shape
        self.max_samples_ = min(max_samples, len(self.data_paths_))
        self.cur_sample_ = None
        self.cur_data_index_ = 0
        self.pad_value=pad_value
        self.index = index

    def get_shape(self):
        if (self.index == 1):
          return self.shape_ 
        else:
          return mm.Dims((1, 2))

    def get_data_type(self):
        return mm.DataType.FLOAT32

    def get_sample(self):
        return self.cur_sample_

    def letterbox(self, img):
        src_h, src_w = img.shape[0], img.shape[1]
        dst_h, dst_w = self.shape_.GetDimValue(2), self.shape_.GetDimValue(3)
        ratio = min(dst_h / src_h, dst_w / src_w)
        unpad_h, unpad_w = int(round(src_h * ratio)), int(round(src_w * ratio))
        img = cv2.resize(img, (unpad_w, unpad_h), interpolation=cv2.INTER_LINEAR)
        # padding
        pad_t = int(round((dst_h - unpad_h) / 2))
        pad_b = dst_h - unpad_h - pad_t
        pad_l = int(round((dst_w - unpad_w) / 2))
        pad_r = dst_w - unpad_w - pad_l
        img = cv2.copyMakeBorder(img, pad_t, pad_b, pad_l, pad_r, cv2.BORDER_CONSTANT, value=(self.pad_value,self.pad_value,self.pad_value))
        return img,(dst_h / src_h, dst_w / src_w)
    
    def preprocess_images(self, data_begin: int, data_end: int) -> np.ndarray:
        imgs = []
        for i in range(data_begin, data_end):
            img = cv2.imread(self.data_paths_[i])
            # resize as letterbox
            img, ratio= self.letterbox(img)
            # BGR to RGB, HWC to CHW
            img = img[:, :, ::-1].transpose(2, 0, 1)
            imgs.append(np.ascontiguousarray(img)[np.newaxis,:])
        # batch and normalize
        input = []
        input.append(np.array([[img.shape[0], img.shape[1]]]).astype(np.float32))
        input.append(np.ascontiguousarray(np.concatenate(tuple(imgs), axis=0).astype(dtype = np.float32) / 255.0))
        input.append(np.array([[ratio[0], ratio[1]]]).astype(np.float32))
        return input[self.index]

    def next(self):
        batch_size = self.shape_.GetDimValue(0)
        data_begin = self.cur_data_index_
        data_end = data_begin + batch_size
        if data_end > self.max_samples_:
            return mm.Status(mm.Code.OUT_OF_RANGE, "Data end reached")
        self.cur_sample_ = self.preprocess_images(data_begin, data_end)
        self.cur_data_index_ = data_end
        return mm.Status.OK()

    def reset(self):
        self.cur_sample_ = None
        self.cur_data_index_ = 0
        return mm.Status.OK()


from calendar import c
import magicmind.python.runtime as mm
from magicmind.python.runtime.parser import Parser
import argparse
from calibrator import CalibData

if __name__ == "__main__":
    parser = argparse.ArgumentParser(description = "model calibrartion and build")
    parser.add_argument('--precision', type=str,   default='force_float16', required=True ,help='Quant_mode')
    parser.add_argument('--batch_size', type=int,   default=8, required=True ,help='batch_size')
    parser.add_argument('--shape_mutable', type=str, default="", required=True ,help='shape_mutable')
    parser.add_argument('--datasets_dir', type=str, default="", required=True ,help='datasets_dir')
    parser.add_argument('--onnx_model', type=str, default="", required=True ,help='onnx_model')
    parser.add_argument('--mm_model', type=str, default="", required=True ,help='magicmind model')
    args = parser.parse_args()

    ONNX_MODEL = args.onnx_model
    BATCH_SIZE = args.batch_size
    MM_MODEL = args.mm_model
    DEV_ID = 0
    INPUT_SIZE = (672, 896)

    # 创建MagicMind parser
    parser = Parser(mm.ModelKind.kOnnx)
    # 指定网络输入数据类型
    network = mm.Network()
    assert parser.parse(network, ONNX_MODEL).ok()
    assert network.get_input(0).set_dimension(mm.Dims((BATCH_SIZE, 3, INPUT_SIZE[0], INPUT_SIZE[1]))).ok()
    config = mm.BuilderConfig()
    precision_json_str = '{"precision_config" : { "precision_mode" : "%s" }}'%args.precision
    assert config.parse_from_string(precision_json_str).ok()
    assert config.parse_from_string("{\"opt_config\":{\"type64to32_conversion\":true}}").ok()
    assert config.parse_from_string("{\"opt_config\":{\"conv_scale_fold\":true}}").ok()
    # 禁用模型输入输出规模可变功能
    if args.shape_mutable=='true':
        assert config.parse_from_string('{"graph_shape_mutable": true}').ok()
        assert config.parse_from_string('{"dim_range": {"0": {"min": [1, 3, 672, 672], "max": [32, 3, 1500, 2500]}}}').ok()
    else:
        assert config.parse_from_string('{"graph_shape_mutable": false}').ok()
    assert config.parse_from_string('{"archs":[{"mtp_372": [2, 6, 8]}]}').ok()
    if args.precision != "force_float16" and args.precision != "force_float32":
        # 量化算法，支持对称量化（symmetric)和非对称量化（asymmetric）。当量化统计算法设置为EQNM_ALOGORITHM时，仅适用于对称量化。
        assert config.parse_from_string('{"precision_config": {"activation_quant_algo": "symmetric"}}').ok()
        # 设置量化粒度，支持按tensor量化（per_tensor）和按通道量化（per_axis）两种。
        assert config.parse_from_string('{"precision_config": {"weight_quant_granularity": "per_tensor"}}').ok()

    # 打开设备
    with mm.System() as mm_sys:
        dev_count = mm_sys.device_count()
        dev = mm.Device()
        dev.id = DEV_ID
        assert dev.active().ok()
        if args.precision != "force_float16" and args.precision != "force_float32":
            print("Calibrating...")
            # 样本数据所在目录(目录中需存放若干jpg格式图片)
            CALIB_SAMPLES_DIR = args.datasets_dir
            # 最大样本数量
            MAX_CALIB_SAMPLES = BATCH_SIZE
            # 创建量化工具并设置量化统计算法
            calib_data = CalibData(mm.Dims((BATCH_SIZE, 3, INPUT_SIZE[0], INPUT_SIZE[1])), MAX_CALIB_SAMPLES, CALIB_SAMPLES_DIR)
            calibrator = mm.Calibrator([calib_data])
            assert calibrator is not None
            # 设置量化统计算法，支持线性统计算法（LINEAR_ALGORITHM）及加强的最小化量化噪声算法（EQM_ALGORITHM）。
            assert calibrator.set_quantization_algorithm(mm.QuantizationAlgorithm.LINEAR_ALGORITHM).ok()
            # 进行量化
            assert calibrator.calibrate(network, config).ok()
            print("Calibrate Done!")

        # 生成模型
        builder = mm.Builder()
        assert builder is not None
        mm_model = builder.build_model("model", network, config)
        assert mm_model is not None
        # 将模型序列化为离线文件
        assert mm_model.serialize_to_file(MM_MODEL).ok()
        print("Generate model done, model save to %s" % args.mm_model)



import tensorflow.compat.v1 as tf
from tensorflow.python.grappler import tf_optimizer
from tensorflow.python.training import saver
from tensorflow.core.protobuf import rewriter_config_pb2
from tensorflow.python.framework import ops
import six as _six
from tensorflow.python.framework import graph_io

input_model = "../data/models/roformer_const.pb"
output_model_path = "../data/models/"
output_model_name = "sim_finish.pb"

def _to_bytes(s):
  """Encode s if it is a sequence of chars."""
  if isinstance(s, _six.text_type):
    return s.encode("utf-8", errors="surrogateescape")
  return s

with tf.gfile.FastGFile(input_model, 'rb') as f:
  graph_def = tf.GraphDef()
  graph_def.ParseFromString(f.read())

graph = tf.Graph()
with graph.as_default():
  tf.import_graph_def(graph_def, name="")
  meta_graph_def = saver.export_meta_graph(
    graph_def=graph.as_graph_def(add_shapes=True), graph=graph)

  grappler_session_config = tf.ConfigProto()
  rewriter_config_with_trt = rewriter_config_pb2.RewriterConfig()
  rewriter_config_with_trt.optimizers.extend(
    ["MLUControlOptimizer"])
  rewriter_config_with_trt.meta_optimizer_iterations = (
       rewriter_config_pb2.RewriterConfig.ONE)
  #rewriter_config_with_trt.custom_optimizers.add().name = "pruning"

  grappler_session_config.graph_options.rewrite_options.CopyFrom(rewriter_config_with_trt)

  collection_def = meta_graph_def.collection_def["train_op"]
  blacklist = collection_def.node_list.value
  for i in ["Pooler-Dense/BiasAdd"]:
    if isinstance(i, ops.Tensor):
      blacklist.append(_to_bytes(i.name))
    else:
      blacklist.append(_to_bytes(i))

output_graph =  tf_optimizer.OptimizeGraph(
           grappler_session_config, meta_graph_def, graph_id=b"tf_graph")

graph_io.write_graph(output_graph, output_model_path,name = output_model_name,as_text = False)

print("frozen pb finish ")


import magicmind.python.runtime as mm
from magicmind.python.runtime.parser import Parser
import clip
from torchvision.datasets import CIFAR100
import torch
import numpy as np
import glob
import os 
import cv2
import os

class CalibData(mm.CalibDataInterface):
    def __init__(self, shape: mm.Dims, index: int, max_samples: int):
        super().__init__()
        self.shape_ = shape
        self.cur_sample_ = None
        self.cur_data_index_ = 0
        self.model, self.preprocess = clip.load("ViT-B/32", device="cpu")
        self.cifar100 = CIFAR100(root=os.path.expanduser(os.environ.get('CIFAR100_DATASETS_PATH')), download=True, train=False,transform=self.preprocess)
        self.max_samples_ = max_samples
        self.text_inputs = torch.cat([clip.tokenize(f"a photo of a {c}") for c in self.cifar100.classes]).to("cpu")
        self.index = index

    def get_shape(self):
        if (self.index == 0):
          return self.shape_ 
        else:
          return mm.Dims((100, 77))

    def get_data_type(self):
        if (self.index == 0):
          return mm.DataType.FLOAT32
        else:
          return mm.DataType.INT32

    def get_sample(self):
        return self.cur_sample_
    
    def preprocess_images(self, data_begin: int, data_end: int) -> np.ndarray:
        imgs = []
        for i in range(data_begin,data_end):
            image, class_id = self.cifar100[i]
            imgs.append(image.numpy())
        if (self.index == 0):
            return np.array(imgs)
        else:
            return self.text_inputs.numpy().astype(np.int32)

    def next(self):
        batch_size = self.shape_.GetDimValue(0)
        data_begin = self.cur_data_index_
        data_end = data_begin + batch_size
        if data_end > self.max_samples_:
            return mm.Status(mm.Code.OUT_OF_RANGE, "Data end reached")
        self.cur_sample_ = self.preprocess_images(data_begin, data_end)
        self.cur_data_index_ = data_end
        return mm.Status.OK()

    def reset(self):
        self.cur_sample_ = None
        self.cur_data_index_ = 0
        return mm.Status.OK()



import numpy as np
import collections
import sys

def rewrite_pad_value(input_data, valid_seq_length, max_seq_length):
    assert len(input_data) == max_seq_length
    for i in range(max_seq_length):
        if i >= valid_seq_length:
            input_data[i] = -float(sys.float_info.max)
    return input_data

def postprocess(results, features, max_seq_length):
    """ Split the raw result into individual results """
    single_result = []
    single_start_logits = []
    single_end_logits = []
    for result in results:
        start_logits = result['start_logits']
        end_logits = result['end_logits']
        rowsplit_start_logits = np.split(start_logits, start_logits.shape[0])
        rowsplit_end_logits = np.split(end_logits, end_logits.shape[0])
        single_start_logits.extend(rowsplit_start_logits)
        single_end_logits.extend(rowsplit_end_logits)
    assert len(features) == len(single_start_logits)
    for (feature_index, feature) in enumerate(features):
        unique_id = int(feature.unique_id)
        flat_start_logits = [float(x) for x in single_start_logits[feature_index].flat]
        flat_end_logits = [float(x) for x in single_end_logits[feature_index].flat]
        # rewrite pad value
        valid_seq_length = np.sum(feature.input_mask)
        flat_start_logits = rewrite_pad_value(flat_start_logits, valid_seq_length, max_seq_length)
        flat_end_logits = rewrite_pad_value(flat_end_logits, valid_seq_length, max_seq_length)
        single_result.append({
            'unique_ids': unique_id,
            'start_logits': flat_start_logits,
            'end_logits': flat_end_logits
        })
    return single_result



import torch
import argparse
import numpy as np
from scipy.io.wavfile import write
import magicmind.python.runtime as mm

import sys
import os
import os.path as path
sys.path.append(os.path.join(os.path.abspath(os.path.dirname(__file__)), '..'))
from text2sequence import prepare_input_sequence
sys.path.append("../")
from decoder import load_decoder_input_model, load_melcrop_model
from mm_runner import MMRunner
from logger import Logger

log = Logger()

def get_args():
    parser = argparse.ArgumentParser(description='recognize with your model')
    parser.add_argument('--encoder_magicmind', required=True, help='encoder magicmind model')
    parser.add_argument('--decoder_magicmind', required=True, help='decoder magicmind model')
    parser.add_argument('--postnet_magicmind', required=True, help='postnet magicmind model')
    parser.add_argument('--waveglow_magicmind', required=True, help='waveglow magicmind model')
    parser.add_argument('-s', '--sigma-infer', default=0.6, type=float)
    parser.add_argument('-sr', '--sampling-rate', default=22050, type=int,
                        help='Sampling rate')
    parser.add_argument('--stft-hop-length', type=int, default=256,
                        help='STFT hop length for estimating audio length from mel size')
    parser.add_argument('-il', '--input-length', type=int, default=128,
                        help='Input length')
    parser.add_argument('--batch_size', type=int, default=1,
                        help='batch_size')
    parser.add_argument('--device_id', type=int, default=0,
                        help='The device ordinal to be used. eg: "0,1,2"')
    parser.add_argument('--no-waveglow', default=False, action='store_true',
                        help='The waveglow model will not be tested when --no-waveglow is set.')
    args = parser.parse_args()
    print(args)
    return args

def infer_tacotron2_mm(device, encoder, decoder, postnet,
                       sequences, sequence_lengths, max_sequence_len):

    #encoder
    encoder_outputs = encoder([sequences, sequence_lengths])

    #decoder
    memory = encoder_outputs[0]
    processed_memory = encoder_outputs[1]
    memory_lengths = sequence_lengths
    max_memory_len = max_sequence_len
    
    bs = memory.shape[0]
    seq_len = memory.shape[1]
    attention_rnn_dim = 1024
    decoder_rnn_dim = 1024
    encoder_embedding_dim = 512
    n_mel_channels = 80

    tensor_shapes = [mm.Dims((bs, n_mel_channels)),         # decoder_input
                     mm.Dims((bs, attention_rnn_dim)),      # attention_hidden
                     mm.Dims((bs, attention_rnn_dim)),      # attention_cell
                     mm.Dims((bs, decoder_rnn_dim)),        # decoder_hidden
                     mm.Dims((bs, decoder_rnn_dim)),        # decoder_cell
                     mm.Dims((bs, seq_len)),                # attention_weights
                     mm.Dims((bs, seq_len)),                # attention_weights_cum
                     mm.Dims((bs, encoder_embedding_dim))]  # attention_context
    total_size = 0
    for shape in tensor_shapes:
        total_size += shape.GetElementCount()
    total_size *= 4  # for float32

    init_decoder_input_model = load_decoder_input_model(device)
    output_tensors = init_decoder_input_model([np.array([total_size], dtype=np.int32), memory_lengths, np.array([max_memory_len], dtype=np.int32)])
    mask = output_tensors[1]
    base_output = output_tensors[0]
    decoder_input_tensors = []
    start_index = 0
    for i in range(len(tensor_shapes)):
        end_index = start_index + tensor_shapes[i].GetElementCount()
        tensor=base_output[start_index:end_index]
        tensor=tensor.reshape(tensor_shapes[i].GetDims())
        start_index = end_index
        decoder_input_tensors.append(tensor)
    decoder_input_tensors.append(memory)
    decoder_input_tensors.append(processed_memory)
    decoder_input_tensors.append(mask)
    decoder_outputs = decoder(decoder_input_tensors)

    #postnet
    mel_outputs = decoder_outputs[0]
    melcrop_model = load_melcrop_model(device)
    mel_outputs = melcrop_model([mel_outputs, decoder_outputs[1]])[0]
    postnet_outputs = postnet([mel_outputs])

    mel_lengths = decoder_outputs[1]
    mel_outputs_postnet = postnet_outputs[0]
    return mel_outputs_postnet, mel_lengths

def infer_waveglow_mm(waveglow, mel):
    #waveglow
    mel_size = mel.shape[2]
    batch_size = mel.shape[0]
    stride = 256
    ngroup = 8
    z_size = mel_size * stride
    z_size = z_size // ngroup
    z = torch.randn(batch_size, ngroup, z_size).unsqueeze(3).numpy()
    mel = np.expand_dims(mel, 3)
    audios = waveglow([mel, z])
    return audios[0]

if __name__ == "__main__":
    args = get_args()

    if not os.path.exists(args.encoder_magicmind):
        log.info("please generate encoder model first!!!")
        exit()

    if not os.path.exists(args.decoder_magicmind):
        log.info("please generate decoder model first!!!")
        exit()

    if not os.path.exists(args.postnet_magicmind):
        log.info("please generate postnet model first!!!")
        exit()

    if not os.path.exists(args.waveglow_magicmind):
        log.info("please generate waveglow model first!!!")
        exit()

    # encoder 
    encoder = MMRunner(mm_file=args.encoder_magicmind, device_id=args.device_id)

    # decoder
    decoder = MMRunner(mm_file=args.decoder_magicmind, device_id=args.device_id)

    # postnet
    postnet = MMRunner(mm_file=args.postnet_magicmind, device_id=args.device_id)

    # waveglow
    waveglow = MMRunner(mm_file=args.waveglow_magicmind, device_id=args.device_id)

    texts = ["The forms of printed letters should be beautiful, and that their arrangement on the page should be reasonable and a help to the shapeliness of the letters themselves. The forms of printed letters should be beautiful, and that their arrangement on the page should be reasonable and a help to the shapeliness of the letters themselves."]
    texts = [texts[0][:args.input_length]]
    texts = texts * args.batch_size

    sequences, sequence_lengths = prepare_input_sequence(texts)
    batch_size = len(sequence_lengths)
    max_sequence_len = sequence_lengths[0]

    mel, mel_lengths = infer_tacotron2_mm(args.device_id, encoder, decoder, postnet, sequences, sequence_lengths, max_sequence_len)
    if not args.no_waveglow:
        audios = infer_waveglow_mm(waveglow, mel)

    num_mels = mel.shape[0] * mel.shape[2]

    if not args.no_waveglow:
        num_samples = audios.shape[0] * audios.shape[1]
        mel_lengths = mel_lengths.squeeze(1)
        #storage
        for i, audio in enumerate(audios):
            audio_path = "audio_"+str(i)+".wav"
            write(audio_path, args.sampling_rate,
                  audio[:mel_lengths[i]*args.stft_hop_length])



# -*- coding:utf-8 -*-
import cv2
import time
import os
import numpy as np
import matplotlib.pyplot as plt
from pse import pse
import argparse
import json

import magicmind.python.runtime as mm

class MagicmindModel(object):
    def __init__(self, model_path, device_id=0):
        self.model = mm.Model()
        self.model.deserialize_from_file(model_path)
        self.dev = mm.Device()
        self.dev.id = device_id
        assert self.dev.active().ok()
        self.engine = self.model.create_i_engine()
        self.context = self.engine.create_i_context()
        self.queue = self.dev.create_queue()
        self.inputs = self.context.create_inputs()
        self.outputs = []
    
    def run_infer(self, data):
        self.inputs[0].from_numpy(data)
        assert self.context.enqueue(self.inputs, self.outputs, self.queue).ok()
        assert self.queue.sync().ok()
        res = self.outputs[0].asnumpy()
        self.outputs = []
        return res

def get_images(test_data_path,nums):
    '''
    find image files in test data path
    :return: list of files found
    '''
    files = []
    exts = ['jpg', 'png', 'jpeg', 'JPG']
    for parent, dirnames, filenames in os.walk(test_data_path):
        for filename in filenames:
            for ext in exts:
                if filename.endswith(ext):
                    files.append(os.path.join(parent, filename))
                    break
    # print('Find {} images'.format(len(sfiles)))
    return files[:nums]


def resize_image(im, max_side_len=1200):
    '''
    resize image to a size multiple of 32 which is required by the network
    :param im: the resized image
    :param max_side_len: limit of max image size to avoid out of memory in gpu
    :return: the resized image and the resize ratio
    '''
    h, w, _ = im.shape

    resize_w = w
    resize_h = h

    # limit the max side
    if max(resize_h, resize_w) > max_side_len:
        ratio = float(max_side_len) / resize_h if resize_h > resize_w else float(max_side_len) / resize_w
    else:
        ratio = 1.

    resize_h = int(resize_h * ratio)
    resize_w = int(resize_w * ratio)

    resize_h = resize_h if resize_h % 32 == 0 else (resize_h // 32 + 1) * 32
    resize_w = resize_w if resize_w % 32 == 0 else (resize_w // 32 + 1) * 32
    im = cv2.resize(im, (int(resize_w), int(resize_h)))

    ratio_h = resize_h / float(h)
    ratio_w = resize_w / float(w)

    return im, (ratio_h, ratio_w)


def detect(seg_maps, timer, image_w, image_h, min_area_thresh=10, seg_map_thresh=0.9, ratio = 1):
    '''
    restore text boxes from score map and geo map
    :param seg_maps:
    :param timer:
    :param min_area_thresh:
    :param seg_map_thresh: threshhold for seg map
    :param ratio: compute each seg map thresh
    :return:
    '''
    if len(seg_maps.shape) == 4:
        seg_maps = seg_maps[0, :, :, ]
    #get kernals, sequence: 0->n, max -> min
    kernals = []
    one = np.ones_like(seg_maps[..., 0], dtype=np.uint8)
    zero = np.zeros_like(seg_maps[..., 0], dtype=np.uint8)
    thresh = seg_map_thresh
    for i in range(seg_maps.shape[-1]-1, -1, -1):
        kernal = np.where(seg_maps[..., i]>thresh, one, zero)
        kernals.append(kernal)
        thresh = seg_map_thresh*ratio
    start = time.time()
    mask_res, label_values = pse(kernals, min_area_thresh)
    timer['pse'] = time.time()-start
    mask_res = np.array(mask_res)
    mask_res_resized = cv2.resize(mask_res, (image_w, image_h), interpolation=cv2.INTER_NEAREST)
    boxes = []
    for label_value in label_values:
        #(y,x)
        points = np.argwhere(mask_res_resized==label_value)
        points = points[:, (1,0)]
        rect = cv2.minAreaRect(points)
        box = cv2.boxPoints(rect)
        boxes.append(box)

    return np.array(boxes), kernals, timer

def show_score_geo(color_im, kernels, im_res):
    fig = plt.figure()
    cmap = plt.cm.hot
    #
    ax = fig.add_subplot(241)
    im = kernels[0]*255
    ax.imshow(im)

    ax = fig.add_subplot(242)
    im = kernels[1]*255
    ax.imshow(im, cmap)

    ax = fig.add_subplot(243)
    im = kernels[2]*255
    ax.imshow(im, cmap)

    ax = fig.add_subplot(244)
    im = kernels[3]*255
    ax.imshow(im, cmap)

    ax = fig.add_subplot(245)
    im = kernels[4]*255
    ax.imshow(im, cmap)

    ax = fig.add_subplot(246)
    im = kernels[5]*255
    ax.imshow(im, cmap)

    ax = fig.add_subplot(247)
    im = color_im
    ax.imshow(im)

    ax = fig.add_subplot(248)
    im = im_res
    ax.imshow(im)

    fig.show()

def main(argv=None):
    parser = argparse.ArgumentParser()
    parser.add_argument('--magicmind_model', type=str, default='../data/', help='saved .mm model name')
    parser.add_argument('--image_dir', type=str, default='./icdar2015/images/', help='test data path')
    parser.add_argument('--image_num', type=int, default=5, help='img num, default:10')
    parser.add_argument('--device_id', type=int, default=0, help='device id')
    parser.add_argument('--img_size', type=list, default=[704, 1216], help='inference size (pixels)')
    parser.add_argument('--batch_size', type=int, default=1, help='inference batch, default:1') 
    parser.add_argument('--save_img', type=bool, default=False, help='save images, please set save path') 
    parser.add_argument('--save_json', type=bool, default=False, help='save results in json,please set save path') 
    parser.add_argument('--json_path', type=str, default=False, help='save predict results') 
    parser.add_argument('--output_dir', type=str, default='./results/', help='test data path')
    args = parser.parse_args() 

    model = MagicmindModel(args.magicmind_model, device_id=args.device_id)  
    res_dict = dict()     
    im_fn_list = get_images(args.image_dir, args.image_num)
    for im_fn in im_fn_list:
        im = cv2.imread(im_fn)[:, :, ::-1]

        start_time = time.time()
        im_resized, (ratio_h, ratio_w) = resize_image(im)
        h, w, _ = im_resized.shape
        im_resized = im_resized[np.newaxis,:]
        timer = {'net': 0, 'pse': 0}
        start = time.time()
        seg_maps = model.run_infer(im_resized)
        timer['net'] = time.time() - start
        boxes, kernels, timer = detect(seg_maps=seg_maps, timer=timer, image_w=w, image_h=h)

        if boxes is not None:
            boxes = boxes.reshape((-1, 4, 2))
            boxes[:, :, 0] /= ratio_w
            boxes[:, :, 1] /= ratio_h
            h, w, _ = im.shape
            boxes[:, :, 0] = np.clip(boxes[:, :, 0], 0, w)
            boxes[:, :, 1] = np.clip(boxes[:, :, 1], 0, h)

        duration = time.time() - start_time
        # save to file
        if boxes is not None:
            num =0
            res = []
            for i in range(len(boxes)):
                # to avoid submitting errors
                box = boxes[i]
                box = box.astype(np.int32)
                if np.linalg.norm(box[0] - box[1]) < 5 or np.linalg.norm(box[3]-box[0]) < 5:
                    continue
                num += 1                
                cv2.polylines(im[:, :, ::-1], [box.astype(np.int32).reshape((-1, 1, 2))], True, color=(255, 255, 0), thickness=2)
                temp = {"points": [[float(box[0, 0]), float(box[0, 1])],
                                    [float(box[1, 0]), float(box[1, 1])],
                                    [float(box[2, 0]), float(box[2, 1])], 
                                    [float(box[3, 0]), float(box[3, 1])]]}
                res.append(temp)
            image_name = im_fn.split('/')[-1][:-4]
            res_dict[image_name] = res
        if args.save_img:
            img_path = os.path.join(args.output_dir, os.path.basename(im_fn))
            cv2.imwrite(img_path, im[:, :, ::-1])
    if args.save_json:
        with open(args.json_path, 'w+') as f:
            json.dump(res_dict, f)
if __name__ == '__main__':
    main()



import numpy as np
import torch

from mm_runner import MMRunner
from logger import Logger

log = Logger()
class MM_Model(object):
    def __init__(self, model_path):
        #mlu
        model = mm.Model()
        model.deserialize_from_file(model_path)
        
        dev = mm.Device()
        dev.id = 0
        dev.active()
        
        with mm.System():
            econfig = mm.Model.EngineConfig()
            econfig.device_type = "MLU"
            engine = model.create_i_engine(econfig)
            assert engine != None, "Failed to create engine"
            self.queue = dev.create_queue()
            assert self.queue != None
            self.context = engine.create_i_context()
            self.inputs = self.context.create_inputs()

    def forward_db(self, input):
        outputs = []
        host_out = []
        self.inputs[0].from_numpy(input.numpy())
        self.context.enqueue(self.inputs, outputs, self.queue)
        self.queue.sync()
        host_out = torch.tensor(outputs[0].asnumpy())
        return host_out

def main(args):
    dbnet = MMRunner(mm_file=args.magicmind_model, device_id=args.device_id)    
    input = np.random.randint(255,size=(args.batch_size, 3, args.input_height, args.input_width)).astype(np.float32)
    
    outputs = dbnet(input)
    db_output = outputs[0].astype(np.float32)

if __name__ == '__main__':
    parser.add_argument('--magicmind_model', type=str, default = '../../data/models/dbnet_pt_model_force_float32_1_1280_800_model', help='model path')
    parser.add_argument("--device_id", "--device_id", type=int, default=0, help="device_id")
    parser.add_argument('--batch_size', dest = 'batch_size', default = 1, type = int, help = 'input batch size')
    parser.add_argument('--input_width', dest = 'input_width', default = 800, type = int, help = 'model input width')
    parser.add_argument('--input_height', dest = 'input_height', default = 1280, type = int, help = 'model input height')
    args = parser.parse_args()
    if not os.path.exists(args.magicmind_model):
        log.error(args.magicmind_model + " does not exist.")
        exit()
    main(args)



import magicmind.python.runtime as mm
import numpy as np
import glob
import os
import cv2


class CalibData(mm.CalibDataInterface):
    def __init__(
        self,
        shape: mm.Dims,
        max_samples: int,
        img_dir: str,
        new_size: int,
        means: list,
        vars_: list,
    ):
        super().__init__()
        assert os.path.isdir(img_dir)
        self.data_paths_ = glob.glob(img_dir + "/*.jpg")
        t = glob.glob(img_dir + "/*.JPEG")
        self.data_paths_ += t
        self.shape_ = shape
        self.max_samples_ = min(max_samples, len(self.data_paths_))
        self.cur_sample_ = None
        self.cur_data_index_ = 0
        self.new_size = new_size
        self.means = means
        self.std = np.sqrt(vars_)

    def get_shape(self):
        return self.shape_

    def get_data_type(self):
        return mm.DataType.FLOAT32

    def get_sample(self):
        return self.cur_sample_

    def preprocess_images(self, data_begin: int, data_end: int) -> np.ndarray:
        imgs = []
        dst_h, dst_w = self.shape_.GetDimValue(2), self.shape_.GetDimValue(3)
        for i in range(data_begin, data_end):
            img = cv2.imread(self.data_paths_[i])
            # resize to NEW_SIZE
            origin_h, origin_w = img.shape[:2]
            scale = self.new_size / min(origin_h, origin_w)
            img = cv2.resize(
                img,
                (round(scale * origin_w), round(scale * origin_h)),
                interpolation=cv2.INTER_LINEAR,
            )
            # center crop
            roi_y1 = round((img.shape[0] - self.new_size) / 2)
            roi_y2 = roi_y1 + dst_h
            roi_x1 = round((img.shape[1] - self.new_size) / 2)
            roi_x2 = roi_x1 + dst_w
            img = img[roi_y1:roi_y2, roi_x1:roi_x2]
            # mean std
            img = img.astype(np.float32)
            # if self.need_insert_bn:
            #    img -= self.mean
            #    img /= self.std

            # cambricon-note: always use insert_bn
            img -= self.means
            img /= self.std
            # HWC to CHW
            img = img.transpose(2, 0, 1)
            imgs.append(np.ascontiguousarray(img)[np.newaxis, :])
        # batch
        return np.ascontiguousarray(np.concatenate(tuple(imgs), axis=0))

    def next(self):
        batch_size = self.shape_.GetDimValue(0)
        data_begin = self.cur_data_index_
        data_end = data_begin + batch_size
        if data_end > self.max_samples_:
            return mm.Status(mm.Code.OUT_OF_RANGE, "Data end reached")
        self.cur_sample_ = self.preprocess_images(data_begin, data_end)
        self.cur_data_index_ = data_end
        return mm.Status.OK()

    def reset(self):
        self.cur_sample_ = None
        self.cur_data_index_ = 0
        return mm.Status.OK()



import magicmind.python.runtime as mm
from magicmind.python.runtime.parser import Parser
import numpy as np
import glob
import os 
import argparse
import cv2
from calibrator import CalibData

if __name__ == "__main__":
    parser = argparse.ArgumentParser(description = "model calibrartion and build")
    parser.add_argument('--precision', type=str,   default='force_float16', required=True ,help='Quant_mode')
    parser.add_argument('--batch_size', type=int,   default=8, required=True ,help='batch_size')
    parser.add_argument('--shape_mutable', type=str, default="false", required=True ,help='shape_mutable')
    parser.add_argument('--datasets_dir', type=str, default="false", required=True ,help='datasets_dir')
    parser.add_argument('--onnx_model', type=str, default="false", required=True ,help='onnx_model')
    parser.add_argument('--mm_model', type=str, default="", required=True ,help='onnx_model')
    args = parser.parse_args()

    DEV_ID = 0
    PAD_VALUE = 128
    BATCH_SIZE = args.batch_size
    INPUT_SIZE = (608, 608) # h x w
    IMAGE_DIR = args.datasets_dir #'val2017'
    ONNXMODEL = args.onnx_model
    CALIB_SAMPLES_DIR = IMAGE_DIR
    MAX_CALIB_SAMPLES = BATCH_SIZE
    MM_MODEL =args.mm_model

    parser = Parser(mm.ModelKind.kOnnx)
    network = mm.Network()
    assert parser.parse(network, ONNXMODEL).ok()
    assert network.get_input(0).set_dimension(mm.Dims((1, 2))).ok()
    assert network.get_input(1).set_dimension(mm.Dims((BATCH_SIZE, 3, INPUT_SIZE[0], INPUT_SIZE[1]))).ok()
    assert network.get_input(2).set_dimension(mm.Dims((1, 2))).ok()

    config = mm.BuilderConfig()
    precision_json_str = '{"precision_config" : { "precision_mode" : "%s" }}'%args.precision
    assert config.parse_from_string(precision_json_str).ok()

    assert config.parse_from_string("{\"opt_config\":{\"type64to32_conversion\":true}}").ok()
    assert config.parse_from_string("{\"opt_config\":{\"conv_scale_fold\":true}}").ok()
    # 禁用模型输入输出规模可变功能
    if args.shape_mutable=='true':
        assert config.parse_from_string('{"graph_shape_mutable": true}').ok()
        assert config.parse_from_string('{"dim_range": {"1": {"min": [1, 3, 608, 608], "max": [32, 3, 608, 608]}}}').ok()
    else:
        assert config.parse_from_string('{"graph_shape_mutable": false}').ok()
    # 硬件平台
    assert config.parse_from_string("""{"archs": ["mtp_372"]}""").ok()
    # 量化算法，支持对称量化（symmetric)和非对称量化（asymmetric）。当量化统计算法设置为EQNM_ALOGORITHM时，仅适用于对称量化。
    assert config.parse_from_string('{"precision_config": {"activation_quant_algo": "symmetric"}}').ok()
    # 设置量化粒度，支持按tensor量化（per_tensor）和按通道量化（per_axis）两种。
    assert config.parse_from_string('{"precision_config": {"weight_quant_granularity": "per_tensor"}}').ok()
    # 打开设备
    with mm.System() as mm_sys:
        dev_count = mm_sys.device_count()
        print("Device count: ", dev_count)
        if DEV_ID >= dev_count:
            print("Invalid DEV_ID set!")
            abort()
        # 打开MLU设备
        dev = mm.Device()
        dev.id = DEV_ID
        assert dev.active().ok()
        calib_data =[]
        if args.precision != "force_float32" and args.precision != "force_float16":
            print("Calibraing...")
            # 创建量化工具并设置量化统计算法
            calib_data.append(CalibData(mm.Dims((BATCH_SIZE, 3, INPUT_SIZE[0], INPUT_SIZE[1])), 0,MAX_CALIB_SAMPLES, CALIB_SAMPLES_DIR,PAD_VALUE))
            calib_data.append(CalibData(mm.Dims((BATCH_SIZE, 3, INPUT_SIZE[0], INPUT_SIZE[1])), 1, MAX_CALIB_SAMPLES, CALIB_SAMPLES_DIR,PAD_VALUE))
            calib_data.append(CalibData(mm.Dims((BATCH_SIZE, 3, INPUT_SIZE[0], INPUT_SIZE[1])), 2, MAX_CALIB_SAMPLES, CALIB_SAMPLES_DIR,PAD_VALUE))
            calibrator = mm.Calibrator(calib_data)
            assert calibrator is not None
            # 设置量化统计算法，支持线性统计算法（LINEAR_ALGORITHM）及加强的最小化量化噪声算法（EQM_ALGORITHM）。
            assert calibrator.set_quantization_algorithm(mm.QuantizationAlgorithm.LINEAR_ALGORITHM).ok()
            assert calibrator.calibrate(network, config).ok()
            print("Calibra Done!")
        # 生成模型
        builder = mm.Builder()
        assert builder is not None
        mm_model = builder.build_model("magicmind model", network, config)
        assert mm_model is not None
        # 将模型序列化为离线文件
        assert mm_model.serialize_to_file(MM_MODEL).ok()



import magicmind.python.runtime as mm
from magicmind.python.runtime.parser import Parser
import numpy as np
import glob
import os 
import argparse
import cv2

if __name__ == "__main__":
    parser = argparse.ArgumentParser(description = "transformers model calibrartion and build")
    parser.add_argument('--precision', type=str,   default='force_float16', required=True ,help='Quant_mode')
    parser.add_argument('--batch_size', type=int,   default=8, required=True ,help='batch_size')
    parser.add_argument('--shape_mutable', type=str, default="", required=True ,help='shape_mutable')
    parser.add_argument('--pt_model', type=str, default="", required=True ,help='pt_model')
    parser.add_argument('--mm_model', type=str, default="", required=True ,help='mm_model')
    args = parser.parse_args()

    PYTORCH_MODEL_PATH = args.pt_model
    BATCH_SIZE = args.batch_size
    MAX_SEQ_LENGTH=128
    MM_MODEL=args.mm_model
    
    # step1: 创建MagicMind PyTorch parser
    mm_parser = Parser(mm.ModelKind.kPytorch)
    # step2: 设置网络输入数据类型
    mm_parser.set_model_param("pytorch-input-dtypes", [mm.DataType.INT32] * 3)
    # step3: 创建一个空的网络实例
    mm_network = mm.Network()
    # step4: 输入PyTorch模型文件，转换MagicMind网络
    status = mm_parser.parse(mm_network, PYTORCH_MODEL_PATH)
    assert status.ok()

    # 通过json字符串配置Builder参数
    config = mm.BuilderConfig()
    # INT64转INT32
    assert config.parse_from_string("{\"opt_config\":{\"type64to32_conversion\": true}}").ok()
    # 选用float16精度模式，支持float16及float32
    precision_json_str = '{"precision_config" : { "precision_mode" : "%s" }}'%args.precision
    assert config.parse_from_string(precision_json_str).ok()
    if args.shape_mutable=='true':
        assert config.parse_from_string('{"graph_shape_mutable": true}').ok()
        assert config.parse_from_string('{ \
        "dim_range": {  \
        "0": {  \
            "min": [1, 1],  \
            "max": [%d, %d]  \
        },  \
        "1": {  \
            "min": [1, 1],  \
            "max": [%d, %d]  \
        },  \
        "2": {  \
            "min": [1, 1],  \
            "max": [%d, %d]  \
        }  \
        }}' % ((BATCH_SIZE, MAX_SEQ_LENGTH) * 3)).ok()
    else:
        assert config.parse_from_string('{"graph_shape_mutable": false}').ok()
    assert config.parse_from_string('{"archs":[{"mtp_372": [2,6,8]}]}').ok()


    # 设置模型输入形状和数据类型
    for i in range(mm_network.get_input_count()):
        mm_network.get_input(i).set_data_type(mm.DataType.INT32)
        mm_network.get_input(i).set_dimension(mm.Dims((BATCH_SIZE, MAX_SEQ_LENGTH)))

    # 生成模型
    mm_builder = mm.Builder()
    mm_model = mm_builder.build_model("bert", mm_network, config)
    assert mm_model is not None
    assert mm_model.serialize_to_file(MM_MODEL).ok()


import numpy as np
import os 
from PIL import Image
from tqdm import tqdm
import sys
import argparse
sys.path.append(os.path.join(os.getenv("MAGICMIND_CLOUD"), "test"))
from record_result import write_result

VOC_DATASETS_PATH = os.environ.get("VOC_DATASETS_PATH")
PROJ_ROOT_PATH = os.environ.get("PROJ_ROOT_PATH")

IMAGE_NAME_LIST = str(VOC_DATASETS_PATH) + '/VOCdevkit/VOC2012/ImageSets/Segmentation/val.txt'
def get_args():
    parser = argparse.ArgumentParser(description='Calculate the mIOU of voc dataset')
    parser.add_argument("--output_dir", dest = 'output_dir', help = "output dir", default = "../segmentation/segment_caffe/data/output/infer_cpp_force_float32_1", type = str)
    parser.add_argument("--language", dest = "language", help = "language which used to infer model", default = "infer_python", type = str)
    return parser.parse_args()

args = get_args()
class Evaluator:
    def __init__(self, nclasses):
        self.nclasses_ = nclasses
        self.hist_ = np.zeros((nclasses, nclasses))

    def fast_hist(self, pred, gt):
        '''计算混淆矩阵'''
        # 去掉ground truth中的白色边界（255）
        mask = (gt >= 0) & (gt < self.nclasses_)
        hist = np.bincount(self.nclasses_ * gt[mask].astype(int) + pred[mask], minlength=self.nclasses_ ** 2)
        hist = hist.reshape(self.nclasses_, self.nclasses_)
        return hist

    def evaluate(self, pred, gt):
        self.hist_ += self.fast_hist(pred, gt)

    def miou(self):
        # iou = 对角线上的值(即预测正确的像素数量) / 预测结果和ground truth并集像素数量
        intersection = np.diag(self.hist_)
        union = self.hist_.sum(axis=0) + self.hist_.sum(axis=1) - np.diag(self.hist_)
        with np.errstate(divide='ignore', invalid='ignore'):
            iou = np.true_divide(intersection, union)
            iou[iou == np.inf] = 0
            iou = np.nan_to_num(iou)
        miou = np.nanmean(iou)
        return miou

with open(IMAGE_NAME_LIST, 'r') as file:
    image_names = file.read().splitlines()

# VOC segmentation 21类（含背景0）
evaluator = Evaluator(21)
for name in tqdm(image_names, "Evaluating"):
    # 载入ground truth
    gt_file = str(VOC_DATASETS_PATH) + '/VOCdevkit/VOC2012/SegmentationClass/' + name + ".png"
    gt_img = Image.open(gt_file)
    gt = np.array(gt_img).flatten()
    # 载入预测结果
    pred_file = args.output_dir + "/" + name + "_result.binary"
    with open(pred_file, 'rb') as file:
        pred = file.read()
        pred = np.frombuffer(pred, dtype=np.uint8)
    # evaluate
    evaluator.evaluate(pred, gt)

print('mIOU : ', evaluator.miou())
write_result(**{"language": args.language, "dataset": "voc2012", "metric":"miou", "eval": evaluator.miou()})



import argparse
import numpy as np
import six
import tokenization
import magicmind.python.runtime as mm
import warnings
warnings.filterwarnings("ignore")
from preprocess import preprocess
from postprocess import postprocess
import sys
import os
from utils import Record
import collections
from mm_runner import MMRunner
from logger import Logger
import math
import json
logging = Logger()

parser = argparse.ArgumentParser()
parser.add_argument("--device_id", "--device_id", type = int, default = 0, help = "device_id")
parser.add_argument("--magicmind_model", "--magicmind_model", type = str, default = "../data/models/bert_qa_pytorch_model")
parser.add_argument("--squad_json", "--squad_json", type = str, default = "../../../../../../datasets/squadv1.1/dev-v1.1.json")
parser.add_argument('--vocab_file', help='Path to file containing entire understandable vocab', required=True)
parser.add_argument("--batch_size", "--batch_size", type = int, default = 16, help = "batch_size")
parser.add_argument("--max_seq_length", "--max_seq_length", type = int, default = 128, help = "max_seq_length")
parser.add_argument('--max_query_length', help= 'The maximum length of a query in number of tokens. Queries longer than this will be truncated',
                    default=64,
                    type=int)
parser.add_argument('--doc_stride', help='When splitting up a long document into chunks, what stride to take between chunks',
                    default=128,
                    type=int)
parser.add_argument('--max_answer_length', help='The maximum length of an answer that can be generated',
                    default=30,
                    type=int)
parser.add_argument('--n_best_size', help='Total number of n-best predictions to generate in the nbest_predictions.json output file',
                    default=20,
                    type=int)
parser.add_argument("--output_dir", "--output_dir", type = str, default = "../data/jsons/output")
parser.add_argument("--version_2_with_negative", help = 'If true, the SQuAD examples contain some that do not have an answer',
                    default=False,
                    type=bool)
parser.add_argument('--verbose_logging', help='If true, all of the warnings related to data processing will be printed',
                    default=False,
                    type=bool)
RawResult = collections.namedtuple("RawResult",
                                   ["unique_id", "start_logits", "end_logits"])

def _get_best_indexes(logits, n_best_size):
  """Get the n-best logits from a list."""
  index_and_score = sorted(enumerate(logits), key=lambda x: x[1], reverse=True)

  best_indexes = []
  for i in range(len(index_and_score)):
    if i >= n_best_size:
      break
    best_indexes.append(index_and_score[i][0])
  return best_indexes

def get_final_text(pred_text, orig_text, do_lower_case):
  """Project the tokenized prediction back to the original text."""

  # When we created the data, we kept track of the alignment between original
  # (whitespace tokenized) tokens and our WordPiece tokenized tokens. So
  # now `orig_text` contains the span of our original text corresponding to the
  # span that we predicted.
  #
  # However, `orig_text` may contain extra characters that we don't want in
  # our prediction.
  #
  # For example, let's say:
  #   pred_text = steve smith
  #   orig_text = Steve Smith's
  #
  # We don't want to return `orig_text` because it contains the extra "'s".
  #
  # We don't want to return `pred_text` because it's already been normalized
  # (the SQuAD eval script also does punctuation stripping/lower casing but
  # our tokenizer does additional normalization like stripping accent
  # characters).
  #
  # What we really want to return is "Steve Smith".
  #
  # Therefore, we have to apply a semi-complicated alignment heruistic between
  # `pred_text` and `orig_text` to get a character-to-charcter alignment. This
  # can fail in certain cases in which case we just return `orig_text`.

  def _strip_spaces(text):
    ns_chars = []
    ns_to_s_map = collections.OrderedDict()
    for (i, c) in enumerate(text):
      if c == " ":
        continue
      ns_to_s_map[len(ns_chars)] = i
      ns_chars.append(c)
    ns_text = "".join(ns_chars)
    return (ns_text, ns_to_s_map)

  # We first tokenize `orig_text`, strip whitespace from the result
  # and `pred_text`, and check if they are the same length. If they are
  # NOT the same length, the heuristic has failed. If they are the same
  # length, we assume the characters are one-to-one aligned.
  tokenizer = tokenization.BasicTokenizer(do_lower_case=do_lower_case)

  tok_text = " ".join(tokenizer.tokenize(orig_text))

  start_position = tok_text.find(pred_text)
  if start_position == -1:
    if args.verbose_logging:
      logging.info(
          "Unable to find text: '%s' in '%s'" % (pred_text, orig_text))
    return orig_text
  end_position = start_position + len(pred_text) - 1

  (orig_ns_text, orig_ns_to_s_map) = _strip_spaces(orig_text)
  (tok_ns_text, tok_ns_to_s_map) = _strip_spaces(tok_text)

  if len(orig_ns_text) != len(tok_ns_text):
    if args.verbose_logging:
      logging.info("Length not equal after stripping spaces: '%s' vs '%s'",
                      orig_ns_text, tok_ns_text)
    return orig_text

  # We then project the characters in `pred_text` back to `orig_text` using
  # the character-to-character alignment.
  tok_s_to_ns_map = {}
  for (i, tok_index) in six.iteritems(tok_ns_to_s_map):
    tok_s_to_ns_map[tok_index] = i

  orig_start_position = None
  if start_position in tok_s_to_ns_map:
    ns_start_position = tok_s_to_ns_map[start_position]
    if ns_start_position in orig_ns_to_s_map:
      orig_start_position = orig_ns_to_s_map[ns_start_position]

  if orig_start_position is None:
    if args.verbose_logging:
      logging.info("Couldn't map start position")
    return orig_text

  orig_end_position = None
  if end_position in tok_s_to_ns_map:
    ns_end_position = tok_s_to_ns_map[end_position]
    if ns_end_position in orig_ns_to_s_map:
      orig_end_position = orig_ns_to_s_map[ns_end_position]

  if orig_end_position is None:
    if args.verbose_logging:
      logging.info("Couldn't map end position")
    return orig_text

  output_text = orig_text[orig_start_position:(orig_end_position + 1)]
  return output_text

def _compute_softmax(scores):
  """Compute softmax probability over raw logits."""
  if not scores:
    return []

  max_score = None
  for score in scores:
    if max_score is None or score > max_score:
      max_score = score

  exp_scores = []
  total_sum = 0.0
  for score in scores:
    x = math.exp(score - max_score)
    exp_scores.append(x)
    total_sum += x

  probs = []
  for score in exp_scores:
    probs.append(score / total_sum)
  return probs

def write_predictions(all_examples, all_features, all_results, n_best_size,
                      max_answer_length, do_lower_case, output_prediction_file,
                      output_nbest_file, output_null_log_odds_file):
  """Write final predictions to the json file and log-odds of null if needed."""
  logging.info("Writing predictions to: %s" % (output_prediction_file))
  logging.info("Writing nbest to: %s" % (output_nbest_file))

  example_index_to_features = collections.defaultdict(list)
  for feature in all_features:
    example_index_to_features[feature.example_index].append(feature)

  unique_id_to_result = {}
  for result in all_results:
    unique_id_to_result[result.unique_id] = result

  _PrelimPrediction = collections.namedtuple(  # pylint: disable=invalid-name
      "PrelimPrediction",
      ["feature_index", "start_index", "end_index", "start_logit", "end_logit"])

  all_predictions = collections.OrderedDict()
  all_nbest_json = collections.OrderedDict()
  scores_diff_json = collections.OrderedDict()

  for (example_index, example) in enumerate(all_examples):
    features = example_index_to_features[example_index]

    prelim_predictions = []
    # keep track of the minimum score of null start+end of position 0
    score_null = 1000000  # large and positive
    min_null_feature_index = 0  # the paragraph slice with min mull score
    null_start_logit = 0  # the start logit at the slice with min null score
    null_end_logit = 0  # the end logit at the slice with min null score
    for (feature_index, feature) in enumerate(features):
      result = unique_id_to_result[feature.unique_id]
      start_indexes = _get_best_indexes(result.start_logits, n_best_size)
      end_indexes = _get_best_indexes(result.end_logits, n_best_size)
      # if we could have irrelevant answers, get the min score of irrelevant
      if args.version_2_with_negative:
        feature_null_score = result.start_logits[0] + result.end_logits[0]
        if feature_null_score < score_null:
          score_null = feature_null_score
          min_null_feature_index = feature_index
          null_start_logit = result.start_logits[0]
          null_end_logit = result.end_logits[0]
      for start_index in start_indexes:
        for end_index in end_indexes:
          # We could hypothetically create invalid predictions, e.g., predict
          # that the start of the span is in the question. We throw out all
          # invalid predictions.
          if start_index >= len(feature.tokens):
            continue
          if end_index >= len(feature.tokens):
            continue
          if start_index not in feature.token_to_orig_map:
            continue
          if end_index not in feature.token_to_orig_map:
            continue
          if not feature.token_is_max_context.get(start_index, False):
            continue
          if end_index < start_index:
            continue
          length = end_index - start_index + 1
          if length > max_answer_length:
            continue
          prelim_predictions.append(
              _PrelimPrediction(
                  feature_index=feature_index,
                  start_index=start_index,
                  end_index=end_index,
                  start_logit=result.start_logits[start_index],
                  end_logit=result.end_logits[end_index]))

    if args.version_2_with_negative:
      prelim_predictions.append(
          _PrelimPrediction(
              feature_index=min_null_feature_index,
              start_index=0,
              end_index=0,
              start_logit=null_start_logit,
              end_logit=null_end_logit))
    prelim_predictions = sorted(
        prelim_predictions,
        key=lambda x: (x.start_logit + x.end_logit),
        reverse=True)

    _NbestPrediction = collections.namedtuple(  # pylint: disable=invalid-name
        "NbestPrediction", ["text", "start_logit", "end_logit"])

    seen_predictions = {}
    nbest = []
    for pred in prelim_predictions:
      if len(nbest) >= n_best_size:
        break
      feature = features[pred.feature_index]
      if pred.start_index > 0:  # this is a non-null prediction
        tok_tokens = feature.tokens[pred.start_index:(pred.end_index + 1)]
        orig_doc_start = feature.token_to_orig_map[pred.start_index]
        orig_doc_end = feature.token_to_orig_map[pred.end_index]
        orig_tokens = example.doc_tokens[orig_doc_start:(orig_doc_end + 1)]
        tok_text = " ".join(tok_tokens)

        # De-tokenize WordPieces that have been split off.
        tok_text = tok_text.replace(" ##", "")
        tok_text = tok_text.replace("##", "")

        # Clean whitespace
        tok_text = tok_text.strip()
        tok_text = " ".join(tok_text.split())
        orig_text = " ".join(orig_tokens)

        final_text = get_final_text(tok_text, orig_text, do_lower_case)
        if final_text in seen_predictions:
          continue

        seen_predictions[final_text] = True
      else:
        final_text = ""
        seen_predictions[final_text] = True

      nbest.append(
          _NbestPrediction(
              text=final_text,
              start_logit=pred.start_logit,
              end_logit=pred.end_logit))

    # if we didn't inlude the empty option in the n-best, inlcude it
    if args.version_2_with_negative:
      if "" not in seen_predictions:
        nbest.append(
            _NbestPrediction(
                text="", start_logit=null_start_logit,
                end_logit=null_end_logit))
    # In very rare edge cases we could have no valid predictions. So we
    # just create a nonce prediction in this case to avoid failure.
    if not nbest:
      nbest.append(
          _NbestPrediction(text="empty", start_logit=0.0, end_logit=0.0))

    assert len(nbest) >= 1

    total_scores = []
    best_non_null_entry = None
    for entry in nbest:
      total_scores.append(entry.start_logit + entry.end_logit)
      if not best_non_null_entry:
        if entry.text:
          best_non_null_entry = entry

    probs = _compute_softmax(total_scores)

    nbest_json = []
    for (i, entry) in enumerate(nbest):
      output = collections.OrderedDict()
      output["text"] = entry.text
      output["probability"] = probs[i]
      output["start_logit"] = entry.start_logit
      output["end_logit"] = entry.end_logit
      nbest_json.append(output)

    assert len(nbest_json) >= 1

    if not args.version_2_with_negative:
      all_predictions[example.qas_id] = nbest_json[0]["text"]
    else:
      # predict "" iff the null score - the score of best non-null > threshold
      score_diff = score_null - best_non_null_entry.start_logit - (
          best_non_null_entry.end_logit)
      scores_diff_json[example.qas_id] = score_diff
      all_predictions[example.qas_id] = best_non_null_entry.text

    all_nbest_json[example.qas_id] = nbest_json

  with open(output_prediction_file, "w") as writer:
    writer.write(json.dumps(all_predictions, indent=4) + "\n")

  with open(output_nbest_file, "w") as writer:
    writer.write(json.dumps(all_nbest_json, indent=4) + "\n")

  if args.version_2_with_negative:
    with open(output_null_log_odds_file, "w") as writer:
      writer.write(json.dumps(scores_diff_json, indent=4) + "\n")

if __name__ == "__main__":
    args = parser.parse_args()
    if not os.path.exists(args.magicmind_model):
        logging.info("please generate magicmind model first!!!")
        exit()
    model = MMRunner(mm_file=args.magicmind_model, device_id=args.device_id)
 
    input_ids, input_mask, segment_ids, eval_features, eval_examples = preprocess(args.vocab_file,
                args.squad_json, args.batch_size, args.max_seq_length,
                args.max_query_length, args.doc_stride, args.version_2_with_negative)

    zipped_input = zip(input_ids, input_mask, segment_ids)
    
    all_results = []
    inputs = [np.zeros((args.batch_size, args.max_seq_length), dtype=np.int64) for i in range(3)]
    for index, feed in enumerate(zipped_input):
        batch_size = feed[0].shape[0]
        for i, data in enumerate(feed):
          inputs[i][:batch_size, :] = feed[i]
        outputs = model(inputs)
        all_results.append({
            'start_logits': outputs[0][:batch_size, :],
            'end_logits': outputs[1][:batch_size, :]
        })

    raw_results = postprocess(all_results, eval_features, args.max_seq_length)
    results = []
    for result in raw_results:
        results.append(
            RawResult(unique_id=result["unique_ids"],
                      start_logits=result["start_logits"],
                      end_logits=result["end_logits"]))

    output_prediction_file = os.path.join(args.output_dir, "predictions.json")
    output_nbest_file = os.path.join(args.output_dir, "nbest_predictions.json")
    output_null_log_odds_file = os.path.join(args.output_dir, "null_odds.json")

    write_predictions(eval_examples, eval_features, results, args.n_best_size,
                      args.max_answer_length, True, output_prediction_file,
                      output_nbest_file, output_null_log_odds_file)



import os
import numpy as np
import magicmind.python.runtime as mm
from magicmind.python.runtime.parser import Parser
import argparse
from calibrator import CalibData
from typing import List
PROTOTXT = 'deploy_resnext50-32x4d.prototxt'
CAFFEMODEL = 'resnext50-32x4d.caffemodel'
def caffe_parser(args):
    # 创建MagicMind parser
    parser = Parser(mm.ModelKind.kCaffe)
    # 使用parser将Caffe模型文件转换为MagicMind Network实例。
    network = mm.Network()
    assert parser.parse(network, args.caffe_model, args.prototxt).ok()
    # 设置网络输入数据形状
    input_dims = mm.Dims((args.batch_size, 3, args.input_height, args.input_width))
    assert network.get_input(0).set_dimension(input_dims).ok()
    return network

def generate_model_config(args):
    config = mm.BuilderConfig()

    # 指定硬件平台
    assert config.parse_from_string('{"archs":[{"mtp_372": [6,8]}]}').ok()
    # 精度模式
    assert config.parse_from_string('{"precision_config":{"precision_mode":"%s"}}' % args.precision).ok()
    # INT64 转 INT32
    assert config.parse_from_string('{"opt_config":{"type64to32_conversion":true}}').ok()
    assert config.parse_from_string('{"opt_config":{"conv_scale_fold":true}}').ok()
    # 输入数据摆放顺序
    # Caffe模型输入数据顺序为NCHW，如下代码转为NHWC输入顺序。
    # 输入顺序的改变需要同步到推理过程中的网络预处理实现，保证预处理结果的输入顺序与网络输入数据顺序一致。
    # 以下JSON字符串中的0代表改变的是网络第一个输入的数据摆放顺序。1则代表第二个输入，以此类推。
    assert config.parse_from_string('{"convert_input_layout": {"0": {"src": "NCHW", "dst": "NHWC"}}}').ok()
    # 模型输入输出规模可变功能
    if args.shape_mutable == "true":
        assert config.parse_from_string('{"graph_shape_mutable":true}').ok()
        assert config.parse_from_string('{"dim_range": {"0": {"min": [1, 3, 224, 224], "max": [64, 3, 224, 224]}}}').ok()
    else:
        assert config.parse_from_string('{"graph_shape_mutable":false}').ok()
    # 预处理标准化过程（减均值、除标准差: (input - mean) / std）集成到模型中, 推理过程的预处理代码中就不再需要执行标准化相关计算。
    # 以下JSON字符串中键值0代表处理的是网络的第一个输入。mean数组中的值代表输入各通道的均值。var数组中的值代表输入各通道的方差即std²。
    # 预处理标准化过程放入模型中后，在模型生成章节中可把模型输入数据类型设置为UINT8，从而达到减少输入拷贝量的目的。
    # 该功能只适用于首层为卷积的网络。
    need_insert_bn = False
    #MEAN = [103.939, 116.779, 123.68]
    #STD = [1.0, 1.0, 1.0]
    MEAN = [103.52, 116.28, 123.675]
    STD = [57.375, 57.12, 58.395]
    for v in MEAN:
        if v != 0:
            need_insert_bn = True
            break
    if not need_insert_bn:
        for v in STD:
            if v != 1.0:
                need_insert_bn = True
                break
    if need_insert_bn:
        var = list(map(lambda num:num*num, STD))
        assert config.parse_from_string('{"insert_bn_before_firstnode": {"0": {"mean":' + str(MEAN) + \
                                        ', "var": ' + str(var) + '}}}').ok()
    # 量化算法，支持对称量化（symmetric)和非对称量化（asymmetric）。当量化统计算法设置为EQNM_ALOGORITHM时，仅适用于对称量化。
    assert config.parse_from_string('{"precision_config": {"activation_quant_algo": "symmetric"}}').ok()
    # 设置量化粒度，支持按tensor量化（per_tensor）和按通道量化（per_axis）两种。
    assert config.parse_from_string('{"precision_config": {"weight_quant_granularity": "per_tensor"}}').ok()
    return config, need_insert_bn, MEAN, STD

def calibrate(args, network : mm.Network, config : mm.BuilderConfig, need_insert_bn: bool, mean: List[float], std: List[float]):
    # 创建量化工具并设置量化统计算法
    calib_data = CalibData(mm.Dims((args.batch_size, 3, args.input_height, args.input_width)), args.batch_size, args.image_dir, need_insert_bn, mean, std)
    calibrator = mm.Calibrator([calib_data])
    assert calibrator is not None
    # 设置量化统计算法，支持线性统计算法（LINEAR_ALGORITHM）及加强的最小化量化噪声算法（EQM_ALGORITHM）。
    assert calibrator.set_quantization_algorithm(mm.QuantizationAlgorithm.LINEAR_ALGORITHM).ok()

    # 打开设备
    with mm.System() as mm_sys:
        dev_count = mm_sys.device_count()
        print("Device count: ", dev_count)
        if args.device_id >= dev_count:
            print("Invalid device set!")
            abort()
        # 打开MLU设备
        dev = mm.Device()
        dev.id = args.device_id
        assert dev.active().ok()
        print("Wroking on MLU ", args.device_id)
    # 进行量化
    assert calibrator.calibrate(network, config).ok()

def main():
    args = argparse.ArgumentParser()
    args.add_argument("--caffe_model", "--caffe_model", type=str, default="../data/models/"+CAFFEMODEL, help="original caffe_model")
    args.add_argument("--prototxt", "--prototxt", type=str, default="../data/models/"+PROTOTXT, help="original prototxt")
    args.add_argument("--output_model", "--output_model", type=str, default="mm_model", help="save mm model to this path")
    args.add_argument("--image_dir", "--image_dir",  type=str, default="/path/to/modelzoo/datasets/imageNet2012/", help="imagenet val datasets")
    args.add_argument("--precision", "--precision", type=str, default="qint8_mixed_float16", help="qint8_mixed_float16, force_float32, force_float16")
    args.add_argument("--shape_mutable", "--shape_mutable", type=str, default="false", help="whether the mm model is dynamic or static or not")
    args.add_argument('--batch_size', dest = 'batch_size', default = 1,
            type = int, help = 'batch_size')
    args.add_argument('--input_width', dest = 'input_width', default = 224,
            type = int, help = 'model input width')
    args.add_argument('--input_height', dest = 'input_height', default = 224,
            type = int, help = 'model input height')
    args.add_argument('--device_id', dest = 'device_id', default = 0,
            type = int, help = 'device_id')
    args = args.parse_args()
    supported_precision = ['qint8_mixed_float16', 'qint8_mixed_float32', 'force_float16', 'force_float32']
    if args.precision not in supported_precision:
        print('precision [' + args.precision + ']', 'not supported')
        exit()
    
    network = caffe_parser(args)
    config, need_insert_bn, mean, std = generate_model_config(args)
    if args.precision.find('qint') != -1:
        print('do calibrate...')
        calibrate(args, network, config, need_insert_bn, mean, std)

    if not need_insert_bn:
        # 当使用了insert_bn_before_firstnode参数后，不需要设置网络输入数据类型，且生成的网络的输入数据类型为UINT8。
        # 本例程序使用UINT8数据类型作为输入，故此处设置网络输入类型为UINT8。
        network.get_input(0).set_data_type(mm.DataType.UINT8)
    # 生成模型
    print('build model...')
    builder = mm.Builder()
    assert builder is not None
    model = builder.build_model("resnext50_caffe_model", network, config)
    assert model is not None
    # 将模型序列化为离线文件
    assert model.serialize_to_file(args.output_model).ok()
    # 检查模型输入数据类型是否为UINT8
    print("input_dtype of this model:", model.get_input_data_type(0))
    print("Generate model done, model save to %s" % args.output_model)

if __name__ == "__main__":
    main()



import argparse
import magicmind.python.runtime as mm
from magicmind.python.runtime.parser import Parser
from magicmind.python.runtime import DataType
from calibrator import CalibData
import time

# import common components
from framework_parser import PytorchParser
from common_calibrate import common_calibrate
from build_param import get_argparser 
from model_process import (
    extract_params,
    config_network,
    get_builder_config,
    build_and_serialize,
)
from logger import Logger

log = Logger()


def get_network(args):
    parser = PytorchParser(args)
    network = parser.parse()
    return network


def calibrate(args, network: mm.Network, config: mm.BuilderConfig):
    # 创建量化工具并设置量化统计算法
    input_dims = mm.Dims(args.input_dims[0])
    custom_max_samples = 10
    max_samples = max(custom_max_samples, args.input_dims[0][0])
    calib_data = CalibData(
        shape=input_dims, max_samples=max_samples, img_dir=args.image_dir
    )
    common_calibrate(args, network, config, calib_data)


def get_args():
    # get common argparser,here is pytorch_parser
    arg_parser = get_argparser()

    # add custom args belonging to the current net.
    arg_parser.add_argument(
        "--image_dir",
        dest="image_dir",
        type=str,
        default="../../datasets/coco/val2017",
        help="coco2017 datasets",
    )
    return arg_parser.parse_args()


def main():
    begin_time = time.time()
    # get net args
    args = get_args()
    network = get_network(args)
    # configure network, such as input_dim, batch_size ...
    config_args = extract_params("MODEL_CONFIG", args)
    config_network(network, config_args)
    # create build configuration
    builder_args = extract_params("MODEL_BUILDER", args)
    build_config = get_builder_config(builder_args)

    if args.precision.find("qint") != -1:
        log.info("Do calibrate...")
        calibrate(args, network, build_config)
    log.info("Build model...")
    # 生成模型并导出
    model_name = "network"
    # build_and_serialize_params = extract_params("MODEL_BUILD_AND_SERIALIZE", args)
    build_and_serialize_args = extract_params("MODEL_BUILD_AND_SERIALIZE", args)
    build_and_serialize(network, build_config, build_and_serialize_args)

    end_time = time.time()
    log.info("gen_model time cost:{:.3f}s".format(end_time - begin_time))


if __name__ == "__main__":
    main()



# ------------------------------------------------------------------------
# Licensed under the Apache License, Version 2.0 (the "License")
# ------------------------------------------------------------------------

import argparse
import json
import random
import os
from tqdm import tqdm

import cv2
import numpy as np
import torch
from torch.utils.data import DataLoader

import sys
sys.path.append("../export_model/HoiTransformer/")
from datasets import build_dataset
from datasets.hico import hoi_interaction_names as hoi_interaction_names_hico
from datasets.hico import coco_instance_ID_to_name as coco_instance_ID_to_name_hico
from datasets.hoia import hoi_interaction_names as hoi_interaction_names_hoia
from datasets.hoia import coco_instance_ID_to_name as coco_instance_ID_to_name_hoia
from datasets.vcoco import hoi_interaction_names as hoi_interaction_names_vcoco
from datasets.vcoco import coco_instance_ID_to_name as coco_instance_ID_to_name_vcoco
from models import build_model
import util.misc as utils

import magicmind.python.runtime as mm

def get_args_parser():
    parser = argparse.ArgumentParser('Set transformer detector', add_help=False)
    parser.add_argument('--lr', default=1e-4, type=float)
    parser.add_argument('--lr_backbone', default=1e-5, type=float)
    parser.add_argument('--batch_size', default=2, type=int)
    parser.add_argument('--weight_decay', default=1e-4, type=float)
    parser.add_argument('--epochs', default=250, type=int)
    parser.add_argument('--lr_drop', default=200, type=int)
    parser.add_argument('--clip_max_norm', default=0.1, type=float, help='gradient clipping max norm')

    # Backbone.
    parser.add_argument('--backbone', choices=['resnet50', 'resnet101'], required=True,
                        help="Name of the convolutional backbone to use")
    parser.add_argument('--position_embedding', default='sine', type=str, choices=('sine', 'learned'),
                        help="Type of positional embedding to use on top of the image features")

    # Transformer.
    parser.add_argument('--enc_layers', default=6, type=int,
                        help="Number of encoding layers in the transformer")
    parser.add_argument('--dec_layers', default=6, type=int,
                        help="Number of decoding layers in the transformer")
    parser.add_argument('--dim_feedforward', default=2048, type=int,
                        help="Intermediate size of the feedforward layers in the transformer blocks")
    parser.add_argument('--hidden_dim', default=256, type=int,
                        help="Size of the embeddings (dimension of the transformer)")
    parser.add_argument('--dropout', default=0.1, type=float,
                        help="Dropout applied in the transformer")
    parser.add_argument('--nheads', default=8, type=int,
                        help="Number of attention heads inside the transformer's attentions")
    parser.add_argument('--num_queries', default=100, type=int,
                        help="Number of query slots")
    parser.add_argument('--pre_norm', action='store_true')

    # Loss.
    parser.add_argument('--no_aux_loss', dest='aux_loss', action='store_false',
                        help="Disables auxiliary decoding losses (loss at each layer)")
    # Matcher.
    parser.add_argument('--set_cost_class', default=1, type=float,
                        help="Class coefficient in the matching cost")
    parser.add_argument('--set_cost_bbox', default=5, type=float,
                        help="L1 box coefficient in the matching cost")
    parser.add_argument('--set_cost_giou', default=2, type=float,
                        help="giou box coefficient in the matching cost")

    # Loss coefficients.
    parser.add_argument('--dice_loss_coef', default=1, type=float)
    parser.add_argument('--bbox_loss_coef', default=5, type=float)
    parser.add_argument('--giou_loss_coef', default=2, type=float)
    parser.add_argument('--eos_coef', default=0.02, type=float,
                        help="Relative classification weight of the no-object class")

    # Dataset parameters.
    parser.add_argument('--dataset_file', choices=['hico', 'vcoco', 'hoia'], required=True)

    parser.add_argument('--model_path', required=True,
                        help='Path of the model to evaluate.')
    parser.add_argument('--log_dir', default='./',
                        help='path where to save temporary files in test')
    parser.add_argument('--device', default='cuda',
                        help='device to use for training')
    parser.add_argument('--seed', default=42, type=int)
    parser.add_argument('--resume', default='', help='resume from checkpoint')
    parser.add_argument('--start_epoch', default=0, type=int, metavar='N',
                        help='start epoch')
    parser.add_argument('--num_workers', default=0, type=int)

    # Distributed training parameters.
    parser.add_argument('--world_size', default=1, type=int,
                        help='number of distributed processes')
    parser.add_argument('--dist_url', default='env://', help='url used to set up distributed training')

    # Visualization.
    parser.add_argument('--max_to_viz', default=10, type=int, help='number of images to visualize')
    parser.add_argument('--save_image', action='store_true', help='whether to save visualization images')
    return parser


def random_color():
    rdn = random.randint(1, 1000)
    b = int(rdn * 997) % 255
    g = int(rdn * 4447) % 255
    r = int(rdn * 6563) % 255
    return b, g, r


def intersection(box_a, box_b):
    # box: x1, y1, x2, y2
    x1 = max(box_a[0], box_b[0])
    y1 = max(box_a[1], box_b[1])
    x2 = min(box_a[2], box_b[2])
    y2 = min(box_a[3], box_b[3])
    if x1 >= x2 or y1 >= y2:
        return 0.0
    return float((x2 - x1 + 1) * (y2 - y1 + 1))


def IoU(box_a, box_b):
    inter = intersection(box_a, box_b)
    box_a_area = (box_a[2]-box_a[0]+1) * (box_a[3]-box_a[1]+1)
    box_b_area = (box_b[2]-box_b[0]+1) * (box_b[3]-box_b[1]+1)
    union = box_a_area + box_b_area - inter
    return inter / float(max(union, 1))


def triplet_nms(hoi_list):
    hoi_list.sort(key=lambda x: x['h_cls'] * x['o_cls'] * x['i_cls'], reverse=True)
    mask = [True] * len(hoi_list)
    for idx_x in range(len(hoi_list)):
        if mask[idx_x] is False:
            continue
        for idx_y in range(idx_x+1, len(hoi_list)):
            x = hoi_list[idx_x]
            y = hoi_list[idx_y]
            iou_human = IoU(x['h_box'], y['h_box'])
            iou_object = IoU(x['o_box'], y['o_box'])
            if iou_human > 0.5 and iou_object > 0.5 and x['i_name'] == y['i_name'] and x['o_name'] == y['o_name']:
                mask[idx_y] = False
    new_hoi_list = []
    for idx in range(len(mask)):
        if mask[idx] is True:
            new_hoi_list.append(hoi_list[idx])
    return new_hoi_list


def inference_on_data(args, model_path, image_set, max_to_viz=10, test_scale=-1):
    assert image_set in ['train', 'test'], image_set
    epoch = 0
    ####### magicmind ###############
    mmmodel = mm.Model()
    assert mmmodel.deserialize_from_file(model_path)
    engine = mmmodel.create_i_engine()
    context = engine.create_i_context()
    dev = mm.Device()
    dev.id = 0
    queue = dev.create_queue()
    inputs = context.create_inputs()



    dataset_val = build_dataset(image_set=image_set, args=args, test_scale=test_scale)
    sampler_val = torch.utils.data.SequentialSampler(dataset_val)
    data_loader_val = DataLoader(dataset_val, args.batch_size, sampler=sampler_val,
                                 drop_last=False, collate_fn=utils.collate_fn, num_workers=args.num_workers)

    log_dir = os.path.join(args.log_dir, 'log')
    if not os.path.exists(log_dir):
        os.makedirs(log_dir, exist_ok=True)

    assert os.path.exists(log_dir), log_dir
    file_name = 'result_s%03d_e%03d_%s_%s.pkl' \
                % (0 if test_scale == -1 else test_scale, epoch, args.dataset_file, args.backbone)
    file_path = os.path.join(log_dir, file_name)
    if os.path.exists(file_path):
        print('step1: file exists, inference done.')
        return file_path

    p_bar = tqdm(total=max_to_viz)

    idx_batch, result_list = 0, []
    outputs = {}
    for samples, targets in data_loader_val:
        outputs_mlu = []
        idx_batch += 1
        if idx_batch >= max_to_viz:
            break
        id_list = [targets[idx]['image_id'] for idx in range(len(targets))]
        org_sizes = [targets[idx]['org_size'] for idx in range(len(targets))]
        inputs[0].from_numpy(samples.tensors.numpy().astype(np.float32))
        context.enqueue(inputs, outputs_mlu, queue)
        queue.sync()
  
        outputs['action_pred_logits'] = outputs_mlu[4].asnumpy() 
        outputs['object_pred_logits'] = outputs_mlu[2].asnumpy()
        outputs['object_pred_boxes'] = outputs_mlu[3].asnumpy()
        outputs['human_pred_logits'] = outputs_mlu[0].asnumpy()
        outputs['human_pred_boxes'] = outputs_mlu[1].asnumpy()

        action_pred_logits = outputs['action_pred_logits']
        object_pred_logits = outputs['object_pred_logits']
        object_pred_boxes = outputs['object_pred_boxes']
        human_pred_logits = outputs['human_pred_logits']
        human_pred_boxes = outputs['human_pred_boxes']
        result_list.append(dict(
            id_list=id_list,
            org_sizes=org_sizes,
            action_pred_logits=action_pred_logits,
            object_pred_logits=object_pred_logits,
            object_pred_boxes=object_pred_boxes,
            human_pred_logits=human_pred_logits,
            human_pred_boxes=human_pred_boxes,
        ))
        p_bar.update()
    with open(file_path, 'wb') as f:
        torch.save(result_list, f)
    print('step1: inference done.')
    return file_path


def parse_model_result(args, result_path, hoi_th=0.9, human_th=0.5, object_th=0.8, max_to_viz=10):
    assert args.dataset_file in ['hico', 'vcoco', 'hoia'], args.dataset_file
    if args.dataset_file == 'hico':
        num_classes = 91
        num_actions = 118
        top_k = 200
        hoi_interaction_names = hoi_interaction_names_hico
        coco_instance_id_to_name = coco_instance_ID_to_name_hico
    elif args.dataset_file == 'vcoco':
        num_classes = 91
        num_actions = 30
        top_k = 35
        hoi_interaction_names = hoi_interaction_names_vcoco
        coco_instance_id_to_name = coco_instance_ID_to_name_vcoco
    else:
        num_classes = 12
        num_actions = 11
        top_k = 35
        hoi_interaction_names = hoi_interaction_names_hoia
        coco_instance_id_to_name = coco_instance_ID_to_name_hoia

    with open(result_path, 'rb') as f:
        output_list = torch.load(f, map_location='cpu')

    idx_batch, final_hoi_result_list = 0, []
    for outputs in tqdm(output_list):  # batch level
        idx_batch += 1
        if idx_batch >= max_to_viz:
            break
        img_id_list = outputs['id_list']
        org_sizes = outputs['org_sizes']
        action_pred_logits = outputs['action_pred_logits']
        object_pred_logits = outputs['object_pred_logits']
        object_pred_boxes = outputs['object_pred_boxes']
        human_pred_logits = outputs['human_pred_logits']
        human_pred_boxes = outputs['human_pred_boxes']
        assert len(action_pred_logits) == len(img_id_list)

        for idx_img in range(len(action_pred_logits)):
            image_id = img_id_list[idx_img]
            hh, ww = org_sizes[idx_img]

            act_cls = torch.nn.Softmax(dim=1)(torch.from_numpy(action_pred_logits[idx_img])).detach().cpu().numpy()[:, :-1]
            human_cls = torch.nn.Softmax(dim=1)(torch.from_numpy(human_pred_logits[idx_img])).detach().cpu().numpy()[:, :-1]
            object_cls = torch.nn.Softmax(dim=1)(torch.from_numpy(object_pred_logits[idx_img])).detach().cpu().numpy()[:, :-1]
            human_box = torch.from_numpy(human_pred_boxes[idx_img]).detach().cpu().numpy()
            object_box = torch.from_numpy(object_pred_boxes[idx_img]).detach().cpu().numpy()

            keep = (act_cls.argmax(axis=1) != num_actions)
            keep = keep * (human_cls.argmax(axis=1) != 2)
            keep = keep * (object_cls.argmax(axis=1) != num_classes)
            keep = keep * (act_cls > hoi_th).any(axis=1)
            keep = keep * (human_cls > human_th).any(axis=1)
            keep = keep * (object_cls > object_th).any(axis=1)

            human_idx_max_list = human_cls[keep].argmax(axis=1)
            human_val_max_list = human_cls[keep].max(axis=1)
            human_box_max_list = human_box[keep]
            object_idx_max_list = object_cls[keep].argmax(axis=1)
            object_val_max_list = object_cls[keep].max(axis=1)
            object_box_max_list = object_box[keep]
            keep_act_scores = act_cls[keep]

            keep_act_scores_1d = keep_act_scores.reshape(-1)
            top_k_idx_1d = np.argsort(-keep_act_scores_1d)[:top_k]
            box_action_pairs = [(idx_1d // num_actions, idx_1d % num_actions) for idx_1d in top_k_idx_1d]

            hoi_list = []
            for idx_box, idx_action in box_action_pairs:
                # action
                i_box = (0, 0, 0, 0)
                i_cls = keep_act_scores[idx_box, idx_action]
                i_name = hoi_interaction_names[int(idx_action)]
                if i_name in ['__background__']:
                    continue
                # human
                cid = human_idx_max_list[idx_box]
                cx, cy, w, h = human_box_max_list[idx_box]
                cx, cy, w, h = cx * ww, cy * hh, w * ww, h * hh
                h_box = list(map(int, [cx - 0.5 * w, cy - 0.5 * h, cx + 0.5 * w, cy + 0.5 * h]))
                h_cls = human_val_max_list[idx_box]
                h_name = coco_instance_id_to_name[int(cid)]
                # object
                cid = object_idx_max_list[idx_box]
                cx, cy, w, h = object_box_max_list[idx_box]
                cx, cy, w, h = cx * ww, cy * hh, w * ww, h * hh
                o_box = list(map(int, [cx - 0.5 * w, cy - 0.5 * h, cx + 0.5 * w, cy + 0.5 * h]))
                o_cls = object_val_max_list[idx_box]
                o_name = coco_instance_id_to_name[int(cid)]
                if i_cls < hoi_th or h_cls < human_th or o_cls < object_th:
                    continue
                pp = dict(
                    h_box=h_box, o_box=o_box, i_box=i_box, h_cls=float(h_cls), o_cls=float(o_cls),
                    i_cls=float(i_cls), h_name=h_name, o_name=o_name, i_name=i_name,
                )
                hoi_list.append(pp)

            hoi_list = triplet_nms(hoi_list)
            item = dict(image_id=image_id, hoi_list=hoi_list)
            final_hoi_result_list.append(item)
    return final_hoi_result_list


def draw_on_image(args, image_id, hoi_list, image_path):
    img_name = image_id
    assert args.dataset_file in ['hico', 'vcoco', 'hoia'], args.dataset_file
    if args.dataset_file == 'hico':
        if 'train2015' in img_name:
            img_path = './data/hico/images/train2015/%s' % img_name
        elif 'test2015' in img_name:
            img_path = './data/hico/images/test2015/%s' % img_name
        else:
            raise NotImplementedError()
    elif args.dataset_file == 'vcoco':
        if 'train2014' in img_name:
            img_path = './data/vcoco/images/train2014/%s' % img_name
        elif 'val2014' in img_name:
            img_path = './data/vcoco/images/val2014/%s' % img_name
        else:
            raise NotImplementedError()
    else:
        if 'trainval' in img_name:
            img_path = './data/hoia/images/trainval/%s' % img_name
        elif 'test' in img_name:
            img_path = './data/hoia/images/test/%s' % img_name
        else:
            raise NotImplementedError()

    img_result = cv2.imread(img_path, cv2.IMREAD_COLOR)
    for idx_box, hoi in enumerate(hoi_list):
        color = random_color()
        # action
        i_cls, i_name = hoi['i_cls'], hoi['i_name']
        cv2.putText(img_result, '%s:%.4f' % (i_name, i_cls),
                    (10, 50 * idx_box + 50), cv2.FONT_HERSHEY_COMPLEX, 1, color, 2)
        # human
        x1, y1, x2, y2 = hoi['h_box']
        h_cls, h_name = hoi['h_cls'], hoi['h_name']
        cv2.rectangle(img_result, (x1, y1), (x2, y2), color, 2)
        cv2.putText(img_result, '%s:%.4f' % (h_name, h_cls), (x1, y2), cv2.FONT_HERSHEY_COMPLEX, 1, color, 2)
        # object
        x1, y1, x2, y2 = hoi['o_box']
        o_cls, o_name = hoi['o_cls'], hoi['o_name']
        cv2.rectangle(img_result, (x1, y1), (x2, y2), color, 2)
        cv2.putText(img_result, '%s:%.4f' % (o_name, o_cls), (x1, y2), cv2.FONT_HERSHEY_COMPLEX, 1, color, 2)
    if img_result.shape[0] > 640:
        ratio = img_result.shape[0] / 640
        img_result = cv2.resize(img_result, (int(img_result.shape[1] / ratio), int(img_result.shape[0] / ratio)))
    cv2.imwrite(image_path, img_result)


def eval_once(args, model_result_path, hoi_th=0.9, human_th=0.5, object_th=0.8, max_to_viz=10, save_image=False):
    assert args.dataset_file in ['hico', 'vcoco', 'hoia'], args.dataset_file

    hoi_result_list = parse_model_result(
        args=args,
        result_path=model_result_path,
        hoi_th=hoi_th,
        human_th=human_th,
        object_th=object_th,
        max_to_viz=max_to_viz,
    )

    result_file = model_result_path.replace('.pkl', '.json')
    with open(result_file, 'w') as writer:
        for idx_img, item in enumerate(hoi_result_list):
            writer.write(json.dumps(item) + '\n')
            if save_image and idx_img < max_to_viz:
                img_path = '%s/dt_%02d.jpg' % (os.path.dirname(model_result_path), idx_img)
                draw_on_image(args, item['image_id'], item['hoi_list'], image_path=img_path)

    os.system('echo %s >> final_report.txt' % result_file)
    if args.dataset_file == 'hico':
        os.system('python3 tools/eval/eval_hico.py --output_file=%s >> final_report.txt' % result_file)
    elif args.dataset_file == 'vcoco':
        os.system('python3 tools/eval/eval_vcoco.py --output_file=%s >> final_report.txt' % result_file)
    else:
        os.system('python3 ../export_model/HoiTransformer/tools/eval/eval_hoia.py --output_file=%s >> final_report.txt' % result_file)
        os.system('python3 ../export_model/HoiTransformer/tools/eval/eval_hoia.py --output_file=%s' % result_file)
    os.system('echo %s >> final_report.txt' % '%f %f %f\n' % (human_th, object_th, hoi_th))
    print(human_th, object_th, hoi_th, '--------------------above')


def run_and_eval(args, model_path, test_scale, max_to_viz=10, save_image=False):
    model_output_file = inference_on_data(
        args=args,
        model_path=model_path,
        image_set='test',
        test_scale=test_scale,
        max_to_viz=max_to_viz,
    )

    for human_th in [0.0]:
        for object_th in [0.0]:
            for hoi_th in [0.0]:
                eval_once(
                    args=args,
                    model_result_path=model_output_file,
                    hoi_th=hoi_th,
                    human_th=human_th,
                    object_th=object_th,
                    max_to_viz=max_to_viz,
                    save_image=save_image,
                )
    pass


def main():
    parser = get_args_parser()
    args = parser.parse_args()
    print(args)

    scales = [672]
    for test_scale in scales:
        for model_path in [
            args.model_path,
        ]:
            os.system('echo %s >> final_report.txt' % model_path)
            print(model_path)
            run_and_eval(
                args=args,
                model_path=model_path,
                test_scale=test_scale,
                max_to_viz=args.max_to_viz if args.save_image else 200*100,
                save_image=args.save_image,
            )
    print('done')


if __name__ == '__main__':
    main()



import clip
import torch
import sys
import os

if __name__=="__main__":
    PRJ_ROOT_PATH = sys.argv[1]    
    ONNX_MODEL_PATH = os.path.join(PRJ_ROOT_PATH,'data/models/clip.onnx')
    if not os.path.exists(ONNX_MODEL_PATH):
        save_dir = os.path.join(PRJ_ROOT_PATH,'data/models')
        if not os.path.exists(save_dir):
            os.mkdir(save_dir)
        model, preprocess = clip.load("ViT-B/32", device="cpu")
        image = torch.randn(1 ,3 , 224, 224)
        text = torch.randint(1, 64, (3,77))
        torch.onnx.export(model,(image,text),ONNX_MODEL_PATH,verbose=True,opset_version=10)
        print("Export torch model to onnx model sucess!")
    

    
    


import numpy as np
import magicmind.python.runtime as mm

import argparse

class MagicmindModel(object):
    def __init__(self, model_path, device_id=0):
        self.model = mm.Model()
        self.model.deserialize_from_file(model_path)
        self.dev = mm.Device()
        self.dev.id = device_id
        assert self.dev.active().ok()
        self.engine = self.model.create_i_engine()
        self.context = self.engine.create_i_context()
        self.queue = self.dev.create_queue()
        self.inputs = self.context.create_inputs()
        self.outputs = []
    
    def run_infer(self, data):
        # data: detect face data, numpy array, nhwc
        self.inputs[0].from_numpy(data)
        assert self.context.enqueue(self.inputs, self.outputs, self.queue).ok()
        assert self.queue.sync().ok()
        res = self.outputs[0].asnumpy()
        return res

def load_data_npz(npz_path):
    d = np.load(npz_path)
    return d["image"], d["pose"]

def main():    
    parser = argparse.ArgumentParser()
    parser.add_argument('--magicmind_model_1', type=str, default='../model1_fp32_b1.mm', help=' .mm model name')
    parser.add_argument('--magicmind_model_2', type=str, default='../model2_fp32_b1.mm', help=' .mm model name')
    parser.add_argument('--magicmind_model_3', type=str, default='../model3_fp32_b1.mm', help=' .mm model name')
    parser.add_argument('--image_dir', type=str, default='../data/type1/AFLW2000.npz', help=' test data')
    parser.add_argument('--image_num', type=int, default='100', help=' test data nums, max 1969')
    parser.add_argument('--device_id', type=int, default=0, help='device id ')
    args = parser.parse_args()
    # init mm model
    model1 = MagicmindModel(model_path=args.magicmind_model_1, device_id=0)
    model2 = MagicmindModel(args.magicmind_model_2, 0)
    model3 = MagicmindModel(args.magicmind_model_3, 0)

    image,pose = load_data_npz(args.image_dir)
    x_data = []
    y_data = []

    for i in range(0,pose.shape[0]):
        temp_pose = pose[i,:]
        if np.max(temp_pose)<=99.0 and np.min(temp_pose)>=-99.0:
            x_data.append(image[i,:,:,:])
            y_data.append(pose[i,:])
    x_data = np.array(x_data)[:args.image_num]
    y_data = np.array(y_data)[:args.image_num]
    p_data = []
    for x in x_data:
        x = np.expand_dims(x, axis=0)
        res1 = model1.run_infer(x)
        res2 = model2.run_infer(x)
        res3 = model3.run_infer(x)
        res_x = np.concatenate([res1, res2, res3])
        p_result = [[np.mean(res_x[:,0]), np.mean(res_x[:,1]), np.mean(res_x[:,2])]]
        p_data.append(p_result)

    p_data = np.concatenate(p_data) 
    pose_matrix = np.mean(np.abs(p_data-y_data),axis=0)
    MAE = np.mean(pose_matrix)
    yaw = pose_matrix[0]
    pitch = pose_matrix[1]
    roll = pose_matrix[2]
    print("\n")
    print("test on AFLW200 samples: {}/1969, the MAE is:".format(args.image_num))
    print('--------------------------------------------------------------------------------')
    print(' MAE = %3.2f, [yaw,pitch,roll] = [%3.2f, %3.2f, %3.2f]'%(MAE, yaw, pitch, roll))
    print('--------------------------------------------------------------------------------')
        
if __name__ == '__main__':
    main()



# Copyright (c) 2020 Mobvoi Inc. (authors: Binbin Zhang, Xiaoyu Chen, Di Wu)
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# Copyright (c) 2021, NVIDIA CORPORATION.  All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# #     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# Copyright (c) 2022, Cambricon, Inc.  All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# #     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
This script is for testing exported onnx encoder and decoder.
The exported onnx models only support batch offline ASR inference.
It requires a python wrapped c++ ctc decoder.
Please install it by following:
https://github.com/Slyne/ctc_decoder.git
"""
from __future__ import print_function

import argparse
import copy
import logging
import os
import sys
from tqdm import tqdm

import torch
import yaml
from torch.utils.data import DataLoader

from wenet.dataset.dataset import Dataset
from wenet.utils.common import IGNORE_ID
from wenet.utils.file_utils import read_symbol_table
from wenet.utils.config import override_config

from magicmind.python.runtime.parser import Parser
from magicmind.python.runtime import Network, BuilderConfig, Builder, Device, System, Model, Builder
import multiprocessing
import numpy as np

from mm_runner import MMRunner
from logger import Logger

log = Logger()

try:
    from swig_decoders import map_batch, \
        ctc_beam_search_decoder_batch, \
        TrieVector, PathTrie
except ImportError:
    print('Please install ctc decoders first by refering to\n' +
          'https://github.com/Slyne/ctc_decoder.git')
    sys.exit(1)


def get_args():
    parser = argparse.ArgumentParser(description='recognize with your model')
    parser.add_argument("--device_id", "--device_id", type=int, default=0, help="device_id")
    parser.add_argument('--config', required=True, help='config file')
    parser.add_argument('--test_data', required=True, help='test data file')
    parser.add_argument('--data_type',
                        default='raw',
                        choices=['raw', 'shard'],
                        help='train and cv data type')
    parser.add_argument('--dict', required=True, help='dict file')
    parser.add_argument('--encoder_magicmind', required=True, help='encoder magicmind model')
    parser.add_argument('--decoder_magicmind', required=True, help='decoder magicmind model')
    parser.add_argument('--result_file', required=True, help='asr result file')
    parser.add_argument('--batch_size',
                        type=int,
                        default=32,
                        help='asr result file')
    parser.add_argument('--mode',
                        choices=[
                            'ctc_greedy_search', 'ctc_prefix_beam_search',
                            'attention_rescoring'],
                        default='attention_rescoring',
                        help='decoding mode')
    parser.add_argument('--bpe_model',
                        default=None,
                        type=str,
                        help='bpe model for english part')
    parser.add_argument('--override_config',
                        action='append',
                        default=[],
                        help="override yaml config")
    parser.add_argument('--fp16',
                        action='store_true',
                        help='whether to export fp16 model, default false')
    args = parser.parse_args()
    print(args)
    return args


def main():
    args = get_args()
    if not os.path.exists(args.encoder_magicmind):
        print("!!!encoder:", args.encoder_magicmind)
        log.info("please generate encoder model first!!!")
        exit()
    if not os.path.exists(args.decoder_magicmind):
        log.info("please generate decoder model first!!!")
        exit()

    logging.basicConfig(level=logging.DEBUG,
                        format='%(asctime)s %(levelname)s %(message)s')

    with open(args.config, 'r') as fin:
        configs = yaml.load(fin, Loader=yaml.FullLoader)
    if len(args.override_config) > 0:
        configs = override_config(configs, args.override_config)

    reverse_weight = configs["model_conf"].get("reverse_weight", 0.0)
    symbol_table = read_symbol_table(args.dict)
    test_conf = copy.deepcopy(configs['dataset_conf'])
    test_conf['filter_conf']['max_length'] = 102400
    test_conf['filter_conf']['min_length'] = 0
    test_conf['filter_conf']['token_max_length'] = 102400
    test_conf['filter_conf']['token_min_length'] = 0
    test_conf['filter_conf']['max_output_input_ratio'] = 102400
    test_conf['filter_conf']['min_output_input_ratio'] = 0
    test_conf['speed_perturb'] = False
    test_conf['spec_aug'] = False
    test_conf['shuffle'] = False
    test_conf['sort'] = False
    test_conf['fbank_conf']['dither'] = 0.0
    test_conf['batch_conf']['batch_type'] = "static"
    test_conf['batch_conf']['batch_size'] = args.batch_size

    test_dataset = Dataset(args.data_type,
                           args.test_data,
                           symbol_table,
                           test_conf,
                           args.bpe_model,
                           partition=False)

    test_data_loader = DataLoader(test_dataset, batch_size=None, num_workers=0)

    # encoder 
    encoder_model = MMRunner(mm_file=args.encoder_magicmind, device_id=args.device_id)

    # decoder
    decoder_model = MMRunner(mm_file=args.decoder_magicmind, device_id=args.device_id)

    # Load dict
    vocabulary = []
    char_dict = {}
    with open(args.dict, 'r') as fin:
        for line in fin:
            arr = line.strip().split()
            assert len(arr) == 2
            char_dict[int(arr[1])] = arr[0]
            vocabulary.append(arr[0])
    eos = sos = len(char_dict) - 1
    with torch.no_grad(), open(args.result_file, 'w') as fout:
        for _, batch in tqdm(enumerate(test_data_loader)):
            keys, feats, _, feats_lengths, _ = batch
            feats, feats_lengths = feats.numpy(), feats_lengths.numpy()
            if args.fp16:
                feats = feats.astype(np.float16)
            encoder_outputs = encoder_model([feats, feats_lengths])

            encoder_out, encoder_out_lens, ctc_log_probs, \
                beam_log_probs, beam_log_probs_idx = encoder_outputs

            beam_size = beam_log_probs.shape[-1]
            batch_size = beam_log_probs.shape[0]
            num_processes = min(multiprocessing.cpu_count(), batch_size)
            if args.mode == 'ctc_greedy_search':
                if beam_size != 1:
                    log_probs_idx = beam_log_probs_idx[:, :, 0]
                batch_sents = []
                for idx, seq in enumerate(log_probs_idx):
                    batch_sents.append(seq[0:encoder_out_lens[idx]].tolist())
                hyps = map_batch(batch_sents, vocabulary, num_processes,
                                 True, 0)
            elif args.mode in ('ctc_prefix_beam_search', "attention_rescoring"):
                batch_log_probs_seq_list = beam_log_probs.tolist()
                batch_log_probs_idx_list = beam_log_probs_idx.tolist()
                batch_len_list = encoder_out_lens.tolist()
                batch_log_probs_seq = []
                batch_log_probs_ids = []
                batch_start = []  # only effective in streaming deployment
                batch_root = TrieVector()
                root_dict = {}
                for i in range(len(batch_len_list)):
                    num_sent = batch_len_list[i]
                    batch_log_probs_seq.append(
                        batch_log_probs_seq_list[i][0:num_sent])
                    batch_log_probs_ids.append(
                        batch_log_probs_idx_list[i][0:num_sent])
                    root_dict[i] = PathTrie()
                    batch_root.append(root_dict[i])
                    batch_start.append(True)
                score_hyps = ctc_beam_search_decoder_batch(batch_log_probs_seq,
                                                           batch_log_probs_ids,
                                                           batch_root,
                                                           batch_start,
                                                           beam_size,
                                                           num_processes,
                                                           0, -2, 0.99999)
                if args.mode == 'ctc_prefix_beam_search':
                    hyps = []
                    for cand_hyps in score_hyps:
                        hyps.append(cand_hyps[0][1])
                    hyps = map_batch(hyps, vocabulary, num_processes, False, 0)
            if args.mode == 'attention_rescoring':
                ctc_score, all_hyps = [], []
                max_len = 0
                for hyps in score_hyps:
                    cur_len = len(hyps)
                    if len(hyps) < beam_size:
                        hyps += (beam_size - cur_len) * [(-float("INF"), (0,))]
                    cur_ctc_score = []
                    for hyp in hyps:
                        cur_ctc_score.append(hyp[0])
                        all_hyps.append(list(hyp[1]))
                        if len(hyp[1]) > max_len:
                            max_len = len(hyp[1])
                    ctc_score.append(cur_ctc_score)
                if args.fp16:
                    ctc_score = np.array(ctc_score, dtype=np.float16)
                else:
                    ctc_score = np.array(ctc_score, dtype=np.float32)
                hyps_pad_sos_eos = np.ones(
                    (batch_size, beam_size, max_len + 2), dtype=np.int64) * IGNORE_ID
                r_hyps_pad_sos_eos = np.ones(
                    (batch_size, beam_size, max_len + 2), dtype=np.int64) * IGNORE_ID
                hyps_lens_sos = np.ones((batch_size, beam_size), dtype=np.int32)
                k = 0
                for i in range(batch_size):
                    for j in range(beam_size):
                        cand = all_hyps[k]
                        l = len(cand) + 2
                        hyps_pad_sos_eos[i][j][0:l] = [sos] + cand + [eos]
                        r_hyps_pad_sos_eos[i][j][0:l] = [sos] + cand[::-1] + [eos]
                        hyps_lens_sos[i][j] = len(cand) + 1
                        k += 1

                decoder_outputs = decoder_model([encoder_out, encoder_out_lens, hyps_pad_sos_eos, hyps_lens_sos, ctc_score])
                best_index = decoder_outputs[0]

                best_sents = []
                k = 0
                for idx in best_index:
                    cur_best_sent = all_hyps[k: k + beam_size][idx]
                    best_sents.append(cur_best_sent)
                    k += beam_size
                hyps = map_batch(best_sents, vocabulary, num_processes)

            for i, key in enumerate(keys):
                content = hyps[i]
                logging.info('{} {}'.format(key, content))
                fout.write('{} {}\n'.format(key, content))

if __name__ == '__main__':
    main()



import numpy as np
import cv2
import magicmind.python.runtime as mm
from typing import List
import os
import glob
from PIL import Image
from torchvision import transforms

def preprocess(input_image, transpose):
    resize_h, resize_w = (232, 232)
    crop_h, crop_w = (224, 224)
    mean = [0.485, 0.456, 0.406]
    std = [0.229, 0.224, 0.225]
    normalize = transforms.Normalize(mean, std)
    preprocess = transforms.Compose(
        [
            transforms.Resize(resize_h, interpolation=Image.BICUBIC),
            transforms.CenterCrop(crop_h),
            transforms.ToTensor(),
            normalize,
        ]
    )
    input_tensor = preprocess(input_image)
    input_numpy = input_tensor.numpy()
    if transpose:
        input_numpy = np.transpose(input_numpy, (1, 2, 0))
    return input_numpy

class CalibData(mm.CalibDataInterface):
    def __init__(self, shape: mm.Dims, max_samples: int, img_dir: str):
        super().__init__()
        assert os.path.isdir(img_dir)
        self.data_paths_ = glob.glob(img_dir + '/*.jpg')
        t = glob.glob(img_dir + '/*.JPEG')
        self.data_paths_ += t
        self.shape_ = shape
        self.max_samples_ = min(max_samples, len(self.data_paths_))
        self.cur_sample_ = None
        self.cur_data_index_ = 0

    def get_shape(self):
        return self.shape_

    def get_data_type(self):
        return mm.DataType.FLOAT32

    def get_sample(self):
        return self.cur_sample_
    
    def preprocess_images(self, data_begin: int, data_end: int) -> np.ndarray:
        imgs = []
        sample_data = []
        dst_h, dst_w = self.shape_.GetDimValue(2), self.shape_.GetDimValue(3)
        for i in range(data_begin, data_end):
            img = Image.open(self.data_paths_[i])
            if img.mode != "RGB":
                img = img.convert("RGB")
            data = preprocess(img, transpose=False)
            sample_data.append(np.expand_dims(data, 0))

        return np.ascontiguousarray(sample_data)
    
    def next(self):
        batch_size = self.shape_.GetDimValue(0)
        data_begin = self.cur_data_index_
        data_end = data_begin + batch_size
        if data_end > self.max_samples_:
            return mm.Status(mm.Code.OUT_OF_RANGE, "Data end reached")
        self.cur_sample_ = self.preprocess_images(data_begin, data_end)
        self.cur_data_index_ = data_end
        return mm.Status.OK()

    def reset(self):
        self.cur_sample_ = None
        self.cur_data_index_ = 0
        return mm.Status.OK()






import subprocess
import os
import numpy as np
import cv2

BASE_DIR = os.path.dirname(os.path.realpath(__file__))

if subprocess.call(['make', '-C', BASE_DIR]) != 0:  # return value
    raise RuntimeError('Cannot compile pse: {}'.format(BASE_DIR))

def pse(kernals, min_area=5):
    '''
    :param kernals:
    :param min_area:
    :return:
    '''
    from .pse import pse_cpp
    kernal_num = len(kernals)
    if not kernal_num:
        return np.array([]), []
    kernals = np.array(kernals)
    label_num, label = cv2.connectedComponents(kernals[kernal_num - 1].astype(np.uint8), connectivity=4)
    label_values = []
    for label_idx in range(1, label_num):
        if np.sum(label == label_idx) < min_area:
            label[label == label_idx] = 0
            continue
        label_values.append(label_idx)

    pred = pse_cpp(label, kernals, c=6)

    return pred, label_values





import argparse
import magicmind.python.runtime as mm
from magicmind.python.runtime.parser import Parser
from magicmind.python.runtime import DataType
from calibrator import CalibData
import time
import sys
import numpy as np

# import common components
from framework_parser import OnnxParser
from common_calibrate import common_calibrate
from build_param import get_argparser 
from model_process import (
    extract_params,
    config_network,
    get_builder_config,
    build_and_serialize,
)
from logger import Logger

log = Logger()

# mmdetection datasets
sys.path.append('../export_model/mmdetection')
from mmcv import Config, DictAction
from mmdet.utils import compat_cfg
from mmdet.datasets import ( build_dataloader, build_dataset,replace_ImageToTensor )

def is_contain(node_name:str, ops:list):
    for op in ops:
        if node_name.__contains__(op):
            return True
    return False

def get_mmdetecion_data(args,max_samples):
    cfg = Config.fromfile(args.config)
    cfg = compat_cfg(cfg)
    samples_per_gpu = 1
    if isinstance(cfg.data.test, dict):
        cfg.data.test.test_mode = True
        samples_per_gpu = cfg.data.test.pop('samples_per_gpu', 1)
        if samples_per_gpu > 1:
            # Replace 'ImageToTensor' to 'DefaultFormatBundle'
            cfg.data.test.pipeline = replace_ImageToTensor(
                cfg.data.test.pipeline)
    elif isinstance(cfg.data.test, list):
        for ds_cfg in cfg.data.test:
            ds_cfg.test_mode = True
        samples_per_gpu = max(
            [ds_cfg.pop('samples_per_gpu', 1) for ds_cfg in cfg.data.test])
        if samples_per_gpu > 1:
            for ds_cfg in cfg.data.test:
                ds_cfg.pipeline = replace_ImageToTensor(ds_cfg.pipeline)

    # build the dataloader
    dataset = build_dataset(cfg.data.test)
    data_loader = build_dataloader(
        dataset,
        samples_per_gpu= 1,
        workers_per_gpu=2,
        dist=False,
        shuffle=False)
    
    dataset = data_loader.dataset
    sample_idx = 0
    sample_data = []
    for i, data in enumerate(data_loader):
        if sample_idx > max_samples:
            break
        img = data['img'][0].numpy()[0]
        sample_data.append(img)
        sample_idx += 1
    return sample_data

def get_network(args):
    parser = OnnxParser(args)
    network = parser.parse()
    node_list =  network.get_all_nodes_in_network()
    node_types = []
    op_list=['NonMaxSuppression_', 'Sqrt_', 'Mul_'] 

    for _node in node_list:
        node_name = _node.get_node_name()
        node_type = _node.get_node_type()
        node_types.append(node_type)
        if is_contain(node_name,op_list):
            input_count = _node.get_input_count()
            output_count = _node.get_output_count()
            
            # check input
            for _input_idx  in range(input_count):
                _input = _node.get_input(_input_idx)
                _input_dtype = _input.get_data_type()
                if _input_dtype in [ mm.DataType.FLOAT16, mm.DataType.FLOAT32 ]:
                    _node.set_precision(_input_idx,mm.DataType.FLOAT32)
                    
            # check output            
            for _output_idx  in range(output_count):
                _output = _node.get_output(_output_idx)
                _output_dtype = _output.get_data_type()
                if _output_dtype in [ mm.DataType.FLOAT16, mm.DataType.FLOAT32 ]:
                    _node.set_output_type(_output_idx,mm.DataType.FLOAT32)
    return network

def calibrate(args, network: mm.Network, config: mm.BuilderConfig):
    # 创建量化工具并设置量化统计算法
    input_dims = mm.Dims(args.input_dims[0])
    max_samples = args.batch_size[0]
    sample_data = get_mmdetecion_data(args,max_samples)
    
    calib_data = CalibData(shape=input_dims, max_samples=max_samples, sample_data = sample_data)
    common_calibrate(args, network, config, calib_data)

def get_args():
    # get common argparser,here is pytorch_parser
    arg_parser = get_argparser()

    # add custom args belonging to the current net.
    arg_parser.add_argument(
        "--config",
        dest="config",
        type=str,
        default="",
        help="config",
    )

    return arg_parser.parse_args()

def main():
    begin_time = time.time()
    # get net args
    args = get_args()
    network = get_network(args)
    # configure network, such as input_dim, batch_size ...
    config_args = extract_params("MODEL_CONFIG", args)
    config_network(network, config_args)
    # create build configuration
    builder_args = extract_params("MODEL_BUILDER", args)
    build_config = get_builder_config(builder_args)

    if args.precision.find("qint") != -1:
        log.info("Do calibrate...")
        calibrate(args, network, build_config)
    log.info("Build model...")
    # 生成模型并导出
    model_name = "network"
    # build_and_serialize_params = extract_params("MODEL_BUILD_AND_SERIALIZE", args)
    build_and_serialize_args = extract_params("MODEL_BUILD_AND_SERIALIZE", args)
    build_and_serialize(network, build_config, build_and_serialize_args)

    end_time = time.time()
    log.info("gen_model time cost:{:.3f}s".format(end_time - begin_time))

if __name__ == "__main__":
    main()



import time
import json
from collections import defaultdict
import torch
import torch.nn.functional as F
from utils import AverageMeter
import os

# from utils import Record
from mm_runner import MMRunner
from logger import Logger

log = Logger()


def get_video_results(outputs, class_names, output_topk):
    sorted_scores, locs = torch.topk(outputs, k=min(output_topk, len(class_names)))

    video_results = []
    for i in range(sorted_scores.size(0)):
        video_results.append(
            {"label": class_names[locs[i].item()], "score": sorted_scores[i].item()}
        )

    return video_results


def inference(data_loader, model, result_path, class_names, no_average, output_topk):
    log.info("inference begin")
    batch_time = AverageMeter()
    data_time = AverageMeter()
    results = {"results": defaultdict(list)}
    end_time = time.time()
    if not os.path.exists(model):
        log.error(model + "does not exist.")
        exit()
    device_id = 0
    model = MMRunner(model, device_id=device_id)

    end_time = time.time()
    forward_time_list = []
    num_pic = 0
    with torch.no_grad():
        for i, (inputs, targets) in enumerate(data_loader):
            data_time.update(time.time() - end_time)
            video_ids, segments = zip(*targets)

            # infer
            forward_start_time = time.time()
            num_pic = num_pic + inputs.shape[0]
            inputs_np = inputs.numpy()
            inputs_list = [inputs_np]
            output = model(inputs_list)
            outputs = torch.from_numpy(output[0])
            forward_end_time = time.time()
            forward_time = forward_end_time - forward_start_time
            forward_time_list.append(forward_time)
            # post-process
            outputs = F.softmax(outputs, dim=1).cpu()

            for j in range(outputs.size(0)):
                results["results"][video_ids[j]].append(
                    {"segment": segments[j], "output": outputs[j]}
                )

            batch_time.update(time.time() - end_time)
            end_time = time.time()

            log.info(
                "[{}/{}]\t"
                "Time {batch_time.val:.3f} ({batch_time.avg:.3f})\t"
                "Data {data_time.val:.3f} ({data_time.avg:.3f})\t".format(
                    i + 1, len(data_loader), batch_time=batch_time, data_time=data_time
                )
            )
    log.info("############# average inference latency #############")
    log.info("number of pictures: ", num_pic)
    log.info(
        "average inference latency: ", sum(forward_time_list) / len(forward_time_list)
    )
    log.info("inference throughput: ", num_pic / sum(forward_time_list))
    log.info("#####################################################")

    inference_results = {"results": {}}
    if not no_average:
        for video_id, video_results in results["results"].items():
            video_outputs = [
                segment_result["output"] for segment_result in video_results
            ]
            video_outputs = torch.stack(video_outputs)
            average_scores = torch.mean(video_outputs, dim=0)
            inference_results["results"][video_id] = get_video_results(
                average_scores, class_names, output_topk
            )
    else:
        for video_id, video_results in results["results"].items():
            inference_results["results"][video_id] = []
            for segment_result in video_results:
                segment = segment_result["segment"]
                result = get_video_results(
                    segment_result["output"], class_names, output_topk
                )
                inference_results["results"][video_id].append(
                    {"segment": segment, "result": result}
                )

    with result_path.open("w") as f:
        json.dump(inference_results, f)



import os
import numpy as np
import argparse
import sys
import time
from utils import Record
from mm_runner import MMRunner
from logger import Logger
log = Logger()
from metric import Auc
sys.path.append("..")
from gen_model.criteo_reader import RecDataset
import paddle
from paddle.io import DataLoader

parser = argparse.ArgumentParser()
parser.add_argument("--device_id", "--device_id", type=int, default=0, help="device_id")
parser.add_argument("--magicmind_model","--magicmind_model",type=str,default="../data/mm_model/xdeepfm_onnx_model_force_float32_true_512", help="save mm model to this path")
parser.add_argument("--batch_size","--batch_size", type=int,default=512,help="batch_size used for infer")
parser.add_argument("--dataset_dir","--dataset_dir",type=str,default="/nfsdata/datasets/criteo/slot_test_data_full",help="criteo test datasets")
parser.add_argument("--sample_num", "--sample_num", type=int, default=1840617, help="sample number")
parser.add_argument("--result_file","--result_file", type=str,default="../data/output/infer_result.txt", help="result_file")

def create_metrics():
    metrics_list_name = ["auc"]
    auc_metric = Auc("ROC")
    metrics_list = [auc_metric]
    return metrics_list, metrics_list_name

def infer_forward(model, batch, metrics_list):
     batch = [tensor.numpy() for tensor in batch]
     pred = model(batch[1:])
     predict_2d = np.concatenate([1 - pred[0], pred[0]], axis=1)
     metrics_list[0].update(preds=predict_2d, labels=batch[0]) 
     return metrics_list, None


if __name__ == "__main__":
    args = parser.parse_args()
    if not os.path.exists(args.magicmind_model):
        print(args.magicmind_model + " does not exist.")
        exit()
    
    # model 定义
    model = MMRunner(mm_file=args.magicmind_model, device_id=args.device_id)
    record = Record(args.result_file)
    batch_size = args.batch_size
    dataset_dir = args.dataset_dir
    sample_num = args.sample_num
    print_interval = 10
    file_list = [os.path.join(dataset_dir, x) for x in os.listdir(dataset_dir)]
    dataset = RecDataset(file_list)
    place = paddle.set_device('cpu')
    test_dataloader= DataLoader(
                        dataset,
                        batch_size=batch_size,
                        places=place,
                        drop_last=True,
                        num_workers=16)
    
    metric_list, metric_list_name = create_metrics()

    infer_reader_cost = 0.0
    infer_run_cost = 0.0
    start = time.time()
    interval_begin = time.time()
    reader_start = time.time()
    #we will drop the last incomplete batch when dataset size is not divisible by the batch size
    assert any(test_dataloader(
    )), "test_dataloader is null, please ensure batch size < dataset size!"

    for batch_id, batch in enumerate(test_dataloader()):
        infer_reader_cost += time.time() - reader_start
        infer_start = time.time()
        batch_size = len(batch[0])
        metric_list, tensor_print_dict = infer_forward(model, batch, metric_list) 
        infer_run_cost += time.time() - infer_start

        if batch_id % print_interval == 0:
            metric_str = ""
            for metric_id in range(len(metric_list_name)):
                metric_str += (
                    metric_list_name[metric_id] +
                    ": {:.6f},".format(metric_list[metric_id].accumulate())
                )
            log.info(
                    "batch_id: {}, ".format(
                     batch_id) + metric_str + 
                    " avg_reader_cost: {:.5f} sec, avg_batch_cost: {:.5f} sec, avg_samples: {:.5f}, ips: {:.2f} ins/s".
                    format(infer_reader_cost / print_interval, 
                           (infer_reader_cost + infer_run_cost) / print_interval,
                           batch_size, 
                           print_interval * batch_size / (time.time() + 0.0001 - interval_begin)))
            record.write("auc result:", False)
            record.write(metric_str,False)
            interval_begin = time.time()
            infer_reader_cost = 0.0
            infer_run_cost = 0.0
        reader_start = time.time()
        if batch_id * batch_size > sample_num:
            break
    end = time.time()
    log.info('total cost time: {}'.format(end-start))


import argparse
import magicmind.python.runtime as mm
from magicmind.python.runtime.parser import Parser
import time
import numpy as np

from preprocess import preprocess, imagenet_dataset
from calibrator import CalibData

# import common components
from framework_parser import PytorchParser
from common_calibrate import common_calibrate
from build_param import get_argparser
from model_process import (
    extract_params,
    config_network,
    get_builder_config,
    build_and_serialize,
)
from logger import Logger

log = Logger()

def get_network(args):
    parser = PytorchParser(args)
    network = parser.parse()
    output_count = network.get_output_count()
    return network

def get_args():
    # get common argparser,here is onnx_parser
    arg_parser = get_argparser()
    
    # add custom args belonging to the current net.
    arg_parser.add_argument(
        "--image_dir",
        dest="image_dir",
        type=str,
        default="",
        help="ImageNet",
    )
    arg_parser.add_argument(
        "--label_file ",
        dest="label_file",
        type=str,
        default="",
        help="label_file",
    )
    return arg_parser.parse_args()

def calibrate(args, network: mm.Network, config: mm.BuilderConfig):
    # 创建量化工具并设置量化统计算法
    dataset = imagenet_dataset(val_txt =args.label_file, image_file_path = args.image_dir, count=args.batch_size[0])
    sample_data = []
    for data, label in dataset:
        data = preprocess(data, transpose=False, normalization=True)
        sample_data.append(data)
    calib_data = CalibData([np.array(sample_data)])
    common_calibrate(args, network, config, calib_data)

def main():
    # get net args
    begin_time = time.time()

    args = get_args()
    
    network = get_network(args)
    # configure network, such as input_dim, batch_size ...
    config_args = extract_params("MODEL_CONFIG", args)
    config_network(network, config_args)
    # create build configuration
    builder_args = extract_params("MODEL_BUILDER", args)
    build_config = get_builder_config(builder_args)
    
    #临时加一下type64to32_conversion 等正式支持
    assert build_config.parse_from_string('{"opt_config":{"type64to32_conversion": true}}').ok()
    
    if args.precision.find("qint") != -1:
        log.info("Do calibrate...")
        calibrate(args, network, build_config)

    log.info("Build model...")
    # 生成模型并导出
    model_name = "network"
    #build_and_serialize_params = extract_params("MODEL_BUILD_AND_SERIALIZE", args)
    build_and_serialize_args = extract_params("MODEL_BUILD_AND_SERIALIZE", args)
    build_and_serialize(network, build_config, build_and_serialize_args)

    end_time = time.time()
    log.info("gen_model time cost:{:.3f}s".format(end_time - begin_time))

if __name__ == "__main__":
    main()